#------------------------------------------------------------------------------
# CFM library to store all the procedures used by different diagnostic scripts
#
# Mar 2009 - Mike Cheng (mcheng1@cisco.com)
#            Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007-2009 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-03/10/2007
# Title: Store(configure) inputs from user and register with EEM.
#
# Name: cfm_lib.tcl
#
# Desc: Stores all the procedures most of which does the validation of the
#       inputs given by the user
#
#-------------------------------------------------------------------

################################################################################
#
#    Description: debug messages
#
#    IN: trace: trace level - 0 no debug
#        msg: debug messages
#
#    OUT: none
#
################################################################################

proc debug {trace msg {level "1"}} {

    if {$trace >= $level} {
        puts $msg
    }
}

################################################################################
#
#    Description: retrieve row data returned from IOS show command
#
#    IN: ios_command: IOS command to retrieve the row data
#        policy: 1 if it is called from a policy, 0 if it is not
#        cmd: command history
#
#    OUT: return row data in a list
#
################################################################################

proc get_row_data {ios_command policy cmd} {

    global trace 

    set tokenList {}
    upvar $cmd cmd2
    debug $trace "enter get_row_data" 
    debug $trace "policy: $policy" 3 
    if {$policy == 1} {
        set output [ios_enable_command2 $ios_command cmd2]
    } elseif {[catch {exec "$ios_command" } output] != 0} {
        puts "ERROR: $ios_command did not execute properly"
        return $tokenList
    }

    set i 0
    set dash_count 0
    set dash "-------------------------------"
    foreach line [split $output "\n"] {
        # count dash lines
        if {[string first $dash $line] >= 0} {
            incr dash_count
            continue
        }
        # exit on blank line after collecting row data
        if { $dash_count >= 2} {
            set trimLine [string trim $line]
            if {[string length $trimLine]  == 0} {
                 debug $trace "Exit on blank line" 
                 break;
            }
        }
        set wordList [regexp -inline -all -- {\S+} $line]

        if { $dash_count == 2} {
            debug $trace $line 4
            foreach word $wordList {
                debug $trace "Word: $word" 2
                lappend tokenList $word
            }
        }
        incr i
    }
    return $tokenList
}

################################################################################
#
#    Description: retrieve row data by header keys and output returned from IOS
#                 show command
#
#    IN: ios_command: IOS command to retrieve the row data
#        keys: a list of header keys separated by space
#        policy: 1 if it is called from a policy, 0 if it is not
#        cmd: command history 
#
#    OUT: return row data in a list
#
################################################################################
proc get_row_data_by_keys {ios_command keyList policy cmd} {

    global trace

    set tokenList {}
    upvar $cmd cmd2
    debug $trace "enter get_row_data_by_keys"
    debug $trace "policy: $policy"
    if {$policy == 1} {
        set output [ios_enable_command2 $ios_command cmd2]
    } elseif {[catch {exec "$ios_command" } output] != 0} {
        puts "ERROR: $ios_command did not execute properly"
        return $tokenList
    }

    set name [info hostname]
    set match 0

    foreach line [split $output "\n"] {
        if {([regexp "[lindex $keyList 0]" $line match1] == 1) && \
            ([regexp "[lindex $keyList 1]" $line match2] == 1) && \
            ([regexp "[lindex $keyList 2]" $line match3] == 1) } {
            set match 1
        } else {
            if {$match == 1} {
                if {([regexp "$name#" $line match4] != 1) && \
                    ([string length [string trim $line]] != 0)} {
                    debug $trace "Match line:"
                    debug $trace $line
                    set wordList [regexp -inline -all -- {\S+} $line]
                    foreach word $wordList {
                        lappend tokenList $word
                    }
                } else {
                    set match 0
                }
            }
        }
    }
    return $tokenList
}

################################################################################
#
#    Description: Get a list of MA names from the file 
#
#    IN: full path of the file name 
#    OUT: A list of MA names 
#         If the file does not exist or contain empty string, the MA list will
#         be empty.
#         If the file does not exist, this function should display error 
#         message to the console
#         Return 1 is successul.  Otherwise, return 0
#
################################################################################

proc get_ma_list {path ma_list} {

    global trace
    upvar $ma_list ma_list2

    debug $trace "enter get_ma_list"
    set ma_list2 {}
    if {[catch {set fp [open $path r]} err] == 1} {
        puts "ERROR: Failed to open the input file $path which contains the\
              MA names\n"
        return 0 
    }
    set contents [read $fp]
    set data [split $contents "\n"]
    foreach line $data {
       set ma [string trim $line]
       if {[string length $ma] > 0} {
           lappend ma_list2 $ma
       }
    }
    if {[llength $ma_list2] == 0} {
       puts "ERROR: MA list is empty in the input file"
       return 0
    }
    return 1
}

################################################################################
#
#    Description: Get a list of MA names from the domain DB 
#
#    IN: policy: called from policy script [TRUE/FALSE]
#         cmd: command history  
#
#    OUT: A list of MA names
#         If the file does not exist or contain empty string, the MA list will
#         be empty
#         If the file does not exist, this function should display error message
#         on the console
#
################################################################################

proc get_ma_from_db {policy cmd} {

    global trace

    set ios_command "show ethernet cfm domain"
    set keyList {Dir Crosscheck MA}
    upvar $cmd cmd2
    set tokenList [get_row_data_by_keys $ios_command $keyList $policy cmd2]

    set size 9 
    set index 8 
    set ma_list {}
    set len [llength $tokenList]
    debug $trace "# of tokens: $len"
    for { set i 0 } { $i < $len } {incr i $size} {
        set service [lindex $tokenList [expr $i + $index]]
        debug $trace "MA: $service"
        lappend ma_list $service
    }
    return $ma_list

}

################################################################################
#
#    Description: Get a list of MA names from the MPDB                            
#
#    IN:  policy: called from policy script [TRUE/FALSE]     
#         cmd: command history  
#                                                                      
#    OUT: A list of MA names
#         If the file does not exist or contain empty string, the MA list will
#         be empty
#         If the file does not exist, this function should display error message
#         on the console
#
################################################################################
proc get_ma_from_mpdb {policy cmd} {

    global trace

    upvar $cmd cmd2
    set ios_command "show ethernet cfm mpdb"
    set tokenList [get_row_data $ios_command $policy cmd2]

    set size 14
    set index 8
    set ma_list {}
    set len [llength $tokenList]
    debug $trace  "# of tokens: $len" 4
    for { set i 0 } { $i < $len } {incr i $size } {
        set service [lindex $tokenList [expr $i + $index]]
        debug $trace "MA: $service"
        lappend ma_list $service
    }
    return $ma_list
}

################################################################################
#     
#    Description: Validat the MA names in the input files are valid MA names 
#                     
#    IN:  MA List   
#         polcy: called from policy script or not - [1/0]
#         cmd: command history
#
#    OUT: Return 1 if all MA names are valid.  Otherwise, return 0
#      
################################################################################    
proc validate_ma_list {ma_list policy cmd} {

    global trace

    set len [llength $ma_list]
    if {$len == 0} {
        puts "ERROR: No MA names specified in the input file\n"
        return 0
    }
    upvar $cmd cmd2
    ### retrieve a list of MA names from the DB
    set domain_ma_list [get_ma_from_db $policy cmd2] 
    set ma_length [llength $domain_ma_list]
    debug $trace "MA DB length: $ma_length"
    if {$ma_length == 0} {
        puts "MA DB Length: $ma_length"  
        return 0
    }

    ### validate each MA in the MA list can be found in the mpdb
    foreach ma $ma_list {
        set found 0
        set wildcard *
        ### check whether ma is wildcard or not. If it is wildcard, return success
        if {[string compare $ma $wildcard] == 0} {
            debug $trace "match wildcard MA name"
            return 1
        }
        foreach ma_db $domain_ma_list {
            debug $trace "Compare $ma $ma_db" 3
            if {[string compare $ma_db $ma] == 0} {
                debug $trace "Match found: $ma $ma_db"
                set found 1
                break
            }
        }
        if {$found == 0} {
            puts "ERROR: MA name \"$ma\" is invalid, not found in the CFM configuration"
            return 0
        }
    }
    return 1
}

################################################################################
#    Description: Match a MA name from the MA list 
#                 If there is a wild card in the MA list, returns success
#
#    IN: MA name and MA list 
#    OUT: Return 1 if there is match.  Otherwise, return 0 
#
################################################################################
proc match_ma {ma ma_list} {

    global trace

    debug $trace "ma: $ma" 3
    debug $trace "ma_list: $ma_list" 3

    if {[lsearch -exact $ma_list "*"] != -1} {
        debug $trace "wild card matches - *" 3
        return 1
    }

    if {[lsearch $ma_list $ma] != -1 } {
        debug $trace "matches..." 3
        return 1
    }

    debug $trace "No match..." 3
    return 0;
}

################################################################################
#    Description: Get valid and invalid MA list based on the MA list from the
#                 input file and the MA list from the DB
#
#    IN: MA list from the input file
#        Invalid MA list -  call by reference
#        cmd: Command History - call by reference
#
#    OUT: Return valid_ma_list: Valid MA list
#                invalid_ma_list: Invalid MA list
#
################################################################################
proc get_valid_ma_list {ma_list invalid cmd } {

    global trace

    upvar $cmd cmd2
    upvar $invalid invalid_ma_list
    set valid_ma_list {}
    set policy 1

    debug $trace "incoming ma_list: $ma_list"
    set ma_list_from_db [get_ma_from_db $policy cmd2]

    ### if input MA list contains a wild card, return all the MIB names
    ### defined in the DB
    if {[lsearch -exact $ma_list "*"] != -1} {
        debug $trace "wild card matches, returns all MAs defined in the DB"
        return $ma_list_from_db
    }

    ### compare incoming ma_list againt the ma_list_from_db and
    ### generate valid_ma_list and invalid_ma_list
    foreach ma $ma_list {
         set match 0
         foreach ma_db $ma_list_from_db {
             if {[string compare $ma $ma_db] == 0} {
                  set match 1
                  break
             }
         }
         if {$match == 1} {
              lappend valid_ma_list $ma
         } else {
              debug $trace "Invalid MA: $ma"
              lappend invalid_ma_list $ma
         }
    }
    return $valid_ma_list
}

################################################################################
#
#    Description: get MA warning message for shut script  
#
#    IN: None 
#
#    OUT: warning message 
#
################################################################################

proc get_shut_ma_warning_message {} {

    set message "User should specify MAs associated to P2P services with UP\
                 MEPs on local interface without service multiplexing. (i.e.\
                 there is only one service/MEP per interface).  Otherwise the\
                 action of the shut script will impact other services."

    return $message

}


################################################################################
#                                                                               
#    Description: validate_p2p                        
#                                                                               
#    IN:  p2p flag                                                              
#                                                                               
#    OUT:  1 is successful.  Otherwise, return 0 
#                                                                               
################################################################################

proc validate_p2p {p2p} {                                          

    ### convert it to upper case
    set value [string toupper $p2p]

    ### only accept "TRUE".  Otherwise, return warning message
    set err    "WARNING: The CFM shut script is NOT registered.\n\n"
    append err "You entered \"FALSE\" for P2P variable.  This means at least one of\n"
    append err "the conditions is NOT met:\n"
    append err "- Point-to-point services (no multipoint services)\n"
    append err "- Target inteface with Up MEP (no Down MEP)\n"
    append err "- No service multiplexing on target interface\n"
    append err "  (multiple services on target interface not recommended)\n\n"
    append err "Change P2P variable to TRUE if recommended conditions are not\
                met\n"
    append err "and still like to deploy this policy\n\n"
    if {([checkFlag $value] == 1)} {
        puts "ERROR: Invalid P2P variable - $p2p"
        puts "       Enter boolean \[TRUE/FALSE\] for the P2P variable"         
        return 0
    }
    if {($value == "FALSE")} {
        puts $err
        return 0
    }
    return 1
}  

################################################################################
#                                                                               
#    Description: exit_from_policy_script           
#                                                                               
#    IN:  name of the policy script      
#                                                                               
#    OUT:  1 is successful.  Otherwise, return 0                                
#                                                                               
################################################################################

proc exit_from_policy_script {script} {                                                       
    set err "ERROR: Aborting from the policy script: $script"                   
    return 1                                                                    
} 


################################################################################
#
#    Description: get CFM policy usages
#
#    IN:  None
#
#    OUT:  return usage message
#
################################################################################

proc get_cfm_policy_usage { } {

    set message "\n\nUsage: tclsh <disk#>:/<dir_name>/sdiag_cfm.tcl \ 
       \[cfm_ondemand.tcl | cfm_cctimeout.tcl | cfm_shut.tcl |\
        cfm_unshut.tcl | cfm_autotrace.tcl\]"

    return $message
}

################################################################################
#    Description: Return remote port status
#
#    IN: mpid, domain, vlan on the remote side
#        cmd - command history (call by reference)
#    OUT: remote port status
#
################################################################################
proc get_remote_port_status {mpid domain vlan cmd} {
    global trace

    debug $trace "Inside get_remote_port_status" 4
    set port_status_label "MEP interface status"

    upvar $cmd cmd2
    set show_command "show ethernet cfm maintenance-points remote detail mpid\
        $mpid domain $domain vlan $vlan"
    debug $trace $show_command

    set output [ios_enable_command2 $show_command cmd2]
    debug $trace $output 3
    foreach line [split $output "\n"] {
        debug $trace $line 4
        if {[string match $port_status_label* $line]} {
           set data [split $line ":"]
           set port_label [lindex $data 0]
           set port_status [lindex $data 1]
           debug $trace "Port Label: $port_label"
           debug $trace "Port Status: $port_status"
           set result [string trim $port_status]
           return $result
        }
    }

    set port_status $output
    return $port_status
}


################################################################################
#
#    Description: get local mep interface 
#
#    IN:  domain, ma name, vlan 
#         cmd: command history
#
#    OUT: local mep interface 
#         1 is successful.  Otherwise, return 0
#
################################################################################

proc get_local_mep_interface {domain ma vlan interface_name cmd} {

    global trace

    debug $trace "Inside get_local_mep_interface" 3
    upvar $interface_name interface_name2
    upvar $cmd cmd2 

    set show_command [format "show ethernet cfm maintenance-points local mep\
        domain %s" $domain]
    debug $trace $show_command 3

    set policy 1
    set tokenList [get_row_data $show_command $policy cmd2]

    set len [llength $tokenList]
    debug $trace "Token length: $len" 3
    debug $trace "Input Domain: $domain" 3
    debug $trace "Input MA: $ma" 3
    debug $trace "Input Vlan: $vlan" 3

    set domainIndex 1
    set vlanIndex 9
    set interfaceIndex 8
    set maIndex 10
    set columns 13 
    for { set i 0 } { $i < $len } { incr i $columns} {
        set row_mpid [lindex $tokenList  $i]
        debug $trace "mpid: $row_mpid" 3
        set row_vlan [lindex $tokenList [expr $i + $vlanIndex]]
        debug $trace "vlan: $row_vlan" 3
        set row_interface [lindex $tokenList [expr $i + $interfaceIndex]]
        debug $trace "Interface: $row_interface" 3
        set row_ma [lindex $tokenList [expr $i + $maIndex]]
        debug $trace "MA: $row_ma" 3
        if {($row_ma == $ma) && ($row_vlan == $vlan)} {
                debug $trace "Matched.." 3
                debug $trace "Interface: $row_interface" 3
                set interface_name2 $row_interface
                return 1 
        }
    }

    debug $trace "No Matched.." 
    puts "ERROR: Fails to get local interface name by domain: $domain\
                 ma: $ma, vlan: $vlan"
    return 0 
}

################################################################################
#
#    Description: Parsing CFM related triggered SYSLOG message  
#
#    IN: msg: CFM related triggered SYSLOG message 
#        domain: pass by reference
#        MA: passs by reference
#        vlan: pass by reference
#        mpid: passs by reference
#        event: pass by reference 
#
#    OUT: update domain, MA, vlan, mpid and event on return 
#
################################################################################

proc parse_cfm_syslog {msg domain MA vlan mpid event} {

     global trace

     set syslog_msg [split $msg " "]

     upvar $domain cfm_domain
     upvar $MA cfm_MA
     upvar $vlan cfm_vlan
     upvar $mpid cfm_mpid
     upvar $event cfm_event
     for {set a 0} {$a < [llength $syslog_msg]} {incr a} {

         switch -exact -- [lindex $syslog_msg $a] {

            domain {
                set cfm_domain [lindex $syslog_msg [expr $a+1]]
            }

            event {
                set cfm_event2 [lindex $syslog_msg [expr $a+2]]
            }

            MA {
                set cfm_MA [lindex $syslog_msg [expr $a+2]]
            }

            mpid {
                set cfm_mpid [lindex $syslog_msg [expr $a+1]]
            }

            vlan {
                set cfm_vlan [lindex $syslog_msg [expr $a+1]]
            }

            default {
                # do nothing
            }
        }
    }
    ### remove last "." from the event code                                      
    set length [string first "."  $cfm_event2]
    if {$length >=0} {
        set cfm_event [string range $cfm_event2 0 [expr $length -1]]
    } else {
        set cfm_event $cfm_event2
    }
    debug $trace "Domain: $cfm_domain"
    debug $trace "MA: $cfm_MA"
    debug $trace "vlan: $cfm_vlan"
    debug $trace "mpid: $cfm_mpid"
    debug $trace "event: $cfm_event"
   
}

#############################################################################
#
#    Description: Executes "show event manager environment command and parses
#                 the output for configuration corresponding to the inputs
#                 entered by the user and displays them in a format easy for the
#                 user to understand
#
#    IN:  type: name of the script
# 
#    OUT: Stores the parsed result in lists and calls the procedure
#         "displayEnvVariables"
#
#############################################################################

proc get_cfm_eem_variables {type} {

### Initialize list which will store the configured inputs ###

    set cfmOndemand {}
    set cfmCctimeout {}
    set cfmShut {}
    set cfmUnshut {}
    set cfmAutotrace {}

    set cfmOndemandOption "FALSE"
    set cfmCctimeoutOption "FALSE"
    set cfmShutOption "FALSE"
    set cfmUnshutOption "FALSE"
    set cfmAutotraceOption "FALSE"

### check validity of non-interactive inputs and give error message ###

    set msg "\n\nERROR: Incorrect input"
    append msg "\nThe value you entered is: "
    set code 1 

    set err "$msg \"$type\""
    append err "\nEnter either \[cfm_ondemand.tcl | cfm_cctimeout.tcl |\
                cfm_shut.tcl | cfm_unshut.tcl | cfm_autotrace.tcl\]"

    switch -exact -- $type {

        cfm_ondemand.tcl {
            set cfmOndemandOption "TRUE"
        }
        cfm_cctimeout.tcl {
            set cfmCctimeoutOption "TRUE"
        }
        cfm_shut.tcl {
            set cfmShutOption "TRUE"
        }
        cfm_unshut.tcl {
            set cfmUnshutOption "TRUE"
        }
        cfm_autotrace.tcl {
            set cfmAutotraceOption "TRUE"
        }
        -help {
            return 0 
        }
        default {
            puts $err
            return
        }
    }

    if {[catch {exec "show event manager environment"} result] != 0} {
        puts stderr "'show event manager environment' did not execute"
    } else {
        set output [split $result "\n"]

        foreach line $output {
            set envVar [lindex $line 1]
            set value [lindex $line 2]

            if {[regexp {_email} $envVar] == 1} {
                lappend emailParam $envVar?$value
            } else {
                switch -regexp -- $envVar {
                    cfmOndemand {
                        lappend cfmOndemand $envVar?$value
                    }
                    cfmCctimeout {
                        lappend cfmCctimeout $envVar?$value
                    }
                    cfmShut {
                        lappend cfmShut $envVar?$value
                    }
                    cfmUnshut {
                        lappend cfmUnshut $envVar?$value
                    }
                    cfmAutotrace {
                        lappend cfmAutotrace $envVar?$value
                    }
                    default {
                        puts ""
                    }
                }
            }
        }

        proc print_default {} {

            puts "NO INPUTS WERE GIVEN FOR THIS SCENARIO"

        }

        switch -exact -- $type {

            cfm_ondemand.tcl {
                if {[llength $cfmOndemand] != 0} {
                    display_cfm_eem_variables $cfmOndemand \
                           "CFM ON-DEMAND SCENARIO"
                } else {
                   print_default
                }
            }
            cfm_cctimeout.tcl {
                if {[llength $cfmCctimeout] != 0} {
                    display_cfm_eem_variables $cfmCctimeout \
                           "CFM CC-TIMEOUT SCENARIO"
                } else {
                   print_default
                }
            }
            cfm_shut.tcl {
                if {[llength $cfmShut] != 0} {
                    display_cfm_eem_variables $cfmShut \
                           "CFM SHUT SCENARIO"
                } else {
                   print_default
                }
            }
            cfm_unshut.tcl {  
                if {[llength $cfmUnshut] != 0} {      
                    display_cfm_eem_variables $cfmUnshut "CFM UNSHUT SCENARIO"     
                } else {                                                        
                   print_default                                                
                }                                                               
            }  
            cfm_autotrace.tcl { 
                if {[llength $cfmAutotrace] != 0} {
                    display_cfm_eem_variables $cfmAutotrace "CFM AUTOTRACE SCENARIO" 
                } else {                                                        
                   print_default                                                
                }                                                               
            } 
            default {
                set code 0 
            }
        }
    }
    return $code
}

#############################################################################
#
#    Description: Displays all the inputs configured on the router based on the
#                 lists passed to this procedure
#
#    IN:  List that contains inputs related to a particular CFM scenario
#         type: script name
#
#    OUT: Displays all the configured inputs in a format easily readable by the
#         user
#
#############################################################################

proc display_cfm_eem_variables {List type} {

    puts "\nTHE INPUTS GIVEN FOR $type ARE:"

    foreach index $List {
        set pair [split $index ?]
        set key [lindex $pair 0]
        set value [lindex $pair 1]
        switch -regexp -- $key {
            cc {
                puts "\tCC ID : $value"
            }
            MAFile {
                puts "\tMA File name is : $value"
            }
            Command {
                puts "\tCommand history option : $value"
            }
            Config {
                puts "\tConfiguration history option : $value"
            }
            Domain {
                puts "\tDomain name : $value"
            }
            Event {
                puts "\tEvent history option : $value"
            }
            MAname {
                puts "\tMA name : $value"
            }
            Mpid {
                puts "\tMEP id : $value"
            }
            Suppress {
                puts "\tSuppress option : $value"
            }
            Prepend {
                puts "\tPrepend option : $value"
            }
            P2P {
                puts "\tP2P service option : $value"
            }
            Timer { 
                puts "\tTimer option : $value secs"                         
            }   
            from {
                puts "\tFrom ID : $value"
            }
            Notif {
                puts "\tNotification : $value"
            }
            server {
                puts "\tEmail Server : $value"
            }
            to {
                puts "\tTo ID : $value"
            }
            default {
                puts "\tNO CONFIGURATION"
            }
        }
    }
}

################################################################################
#
#    Description: Determines if the traceroute is enabled on the router or not
#
#    IN:  policy: called from policy script, 1 is TRUE.  0 is FALSE 
#         cmd: command history
#         result: show traceroute cache command output
#
#    OUT: Returns "1" if the cache is enabled or "0" if no enabled
#
################################################################################

proc is_traceroute_cache_enabled {policy cmd result} {

    global trace
    upvar $cmd cmd2
    upvar $result output
    set ios_command "show ethernet cfm traceroute-cache" 
    debug $trace "Inside is_traceroute_cache_enabled....."
    if {$policy == 1} {
        set output [ios_enable_command2 $ios_command cmd2]
    } elseif {[catch {exec "$ios_command" } output] != 0} {
        puts "$ios_command did not execute properly"
        return 0 
    }

    set match ""
    if {[regexp {traceroute-cache not enabled} $output match]} {
        return 0
    } else {
        return 1
    }
}

################################################################################
#
#    Description: Display the traceroute configuration if it is configured
#
#    IN: cmd: command history
#
#    OUT: return 1 if it is parsed successful.  Otherwise, return 0 
#
################################################################################

proc display_traceroute_configuration {policy cmd hold_timer size} {

    global trace

    upvar $cmd cmd2
    upvar $hold_timer hold_timer2
    upvar $size size2

    ### default hold-time is 100 minutes
    ### default size is 100
    set hold_timer2 100 
    set size2 100

    set ios_command "show run | inc ethernet cfm traceroute cache"
    if {$policy == 1} {
        set output [ios_enable_command2 $ios_command cmd2]
    } elseif {[catch {exec "$ios_command" } output] != 0} {
        puts "ERROR: $ios_command did not execute properly"
        return 0
    }

    debug $trace "Traceroute configuration: $output" 3

    foreach line [split $output "\n"] {
        debug $trace $line 4
        if {[regexp {([^\t]+) hold-time ([0-9]+)} $line match prefix \
                              hold_timer2]} {
               puts "Traceroute Cache Hold Time: $hold_timer2 mins"
        }
        if {[regexp {([^\t]+) size ([0-9]+)} $line match prefix size2]} {
               puts "\n\nTraceroute Cache Size: $size2"

        }
    }
    return 1
}

################################################################################
#
#    Description: perform_traceroute
#
#    IN:  mac, domain, vlan
#         cmd: command history
#
#    OUT: Display traceroute configuration on the console if it is configured
#         1 is successful.  Otherwise, return 0
#
################################################################################

proc perform_traceroute { mac domain vlan cmd} {

   upvar $cmd cmd2
   set output [ios_enable_command2 [format "traceroute ethernet %s domain %s vlan %s" $mac $domain $vlan] cmd2 ]
   ### validate the result
   # puts "Traceroute ethernet $mac domain $domain vlan $vlan"
   if {[regexp {Traceroute terminated} $output] == 1} {
       puts "ERROR: Traceroute command fails..."
       return 0
   }
   return 1
}

################################################################################
#
#    Description: trace remote MEPs info
#
#    IN:  domain, ma list
#         trace_report - call by reference
#         trace_times - call by reference
#         cmd: command history
#
#    OUT: Display traceroute configuration on the console if it is configured
#
################################################################################

proc trace_remote_meps_info { domain maList trace_report trace_times cmd} {

    global trace

    upvar $cmd cmd2
    set policy 1
    debug $trace "enter trace_remote_meps_info with domain: $domain" 3

    set ios_command  "show ethernet cfm main remote domain $domain"
    set tokenList [get_row_data  $ios_command $policy cmd2]
    set len [llength $tokenList]

    debug $trace "Token length: $len"
    upvar $trace_report report2
    upvar $trace_times times
    append report "\nCFM Domain $domain \n"
    set header 0

    ### print_ma_list contains a list of MA names which already prints
    ### the MA names to to the report
    set print_ma_list {}

    set columns 15
    set mpidIndex 0
    set domainIndex 1
    set macIndex 2
    set maIndex 9
    set vlanIndex 11
    set count 0

    ### Iterate each row from table output from the above IOS command
    for { set i 0 } { $i < $len } {incr i $columns } {
        set ma [lindex $tokenList [expr $i + $maIndex]]
        debug $trace "MA: $ma"

        ### if MA name not found in the specified MA list, skip it
        if {[match_ma $ma $maList] != 1} {
            continue
        }

        ### retrieve mpid, vlan and MAC address for each matched MA
        ### in this domain
        set mpid [lindex $tokenList  [expr $i + $mpidIndex]]
        set vlan [lindex $tokenList [expr $i + $vlanIndex]]
        set mac_address [lindex $tokenList [expr $i + $macIndex]]
        append msg "mpid: $mpid " "vlan: $vlan "\
                   "MAC Address: $mac_address"
        debug $trace $msg 

        ### perform trace route
        perform_traceroute $mac_address $domain $vlan cmd2
        if {[lsearch $print_ma_list $ma] == -1} {
            if {$header == 0} {
                append report2 $report
                incr header
            }
            append report2 "   MA $ma - vlan $vlan \n"
            lappend print_ma_list $ma
        }
        append report2 "      mpid $mpid, mac address $mac_address \n"
        incr times
        incr count
    }

    ### if we send out at least one traceroute in this domain
    debug $trace "Times: $times"
    debug $trace "Count: $count"
}

################################################################################
#
#    Description: Return matched domain list based on MA list
#
#    IN: MA list
#    OUT: matched domain list
#
################################################################################

proc get_domain_list {cmd policy} {                                            

    global trace

    set ios_command "show ethernet cfm domain brief"
    set keyList {Domain Index Services}
    upvar $cmd cmd2
    set tokenList [get_row_data_by_keys $ios_command $keyList $policy cmd2]

    set size 5 
    set index 0 
    set domain_list {}
    set len [llength $tokenList]
    debug $trace "# of tokens: $len" 2
    for { set i 0 } { $i < $len } {incr i $size } {
        set domain [lindex $tokenList [expr $i + $index]]
        debug $trace "Domain: $domain" 2
        lappend domain_list $domain
    }
    return $domain_list
}

################################################################################
#
#    Description: Auto trace all end points per domain list
#
#    IN: domain list
#
#    OUT: None
#
################################################################################

proc traceroute_all_mpid { domainList maList filename summary details cmd} {

    global trace

    debug $trace "Domain List: $domainList"
    set traceroute_times 0
    upvar $details traceReport
    upvar $summary message
    upvar $cmd cmd2
    foreach domain $domainList {
        trace_remote_meps_info $domain $maList traceReport traceroute_times cmd
    }
    set msg1 "SERV-DIAG_CFM_autotrace: A total of $traceroute_times MEPS for\
              valid MAs specified in $filename have been automatically\
              tracerouted.  Results stored in the CFM traceroute cache."
    set msg2 "SERV-DIAG_CFM_autotrace: No MEPs are found for MAs specified in\
              $filename.  As a result, no traceroute is executed."
    if {$traceroute_times > 0} {
        append message $msg1
    } else {
        append message $msg2
    }
}

################################################################################
#
#    Description: Get domain name and truncate if nessary 
#
#    IN: Domain name 
#    OUT: Trimmed domain name if applicable 
#
################################################################################

proc get_trimmed_domain { domain } {

    ### truncate domain name after 16 chars
    set domain2 ""
    if {[string length $domain] <= 16} {
        set domain2 $domain
    } else {
        ### take the first 16 chars of the domain name
        set domain2 [string range $domain 0 15]
        append domain2 ".."
    }
    return $domain2
}

###############################################################################
#    Description: Matches a domain name against the domain list that is
#                 obtained by parsing the domain db, and returns 1 if the
#                 domain matches otherwise returns 0
#
#    IN: domain name, cmd history and policy flags
#    OUT: Returns 1 if there is a match.  Otherwise, returns 0
#
###############################################################################

proc validate_ma_name {domain ma_name policy} {

    if {[get_domain_details $domain $ma_name $policy cmd vlan level] == 0} {
        puts "ERROR: The MA name that you specified is not found under the\
              domain \"$domain\"\n"
        return 0
    } else {
        return 1
    } 

}

###############################################################################
#    Description: Matches a domain name against the domain list that is
#                 obtained by parsing the domain db, and returns 1 if the
#                 domain matches otherwise returns 0
#
#    IN: domain name, cmd history and policy flags
#    OUT: Returns 1 if there is a match.  Otherwise, returns 0
#
###############################################################################

proc validate_domain { domain cmd policy } {

    set domain_list {}
    set domain_list [get_domain_list $cmd $policy]
    ### Display error message when the domain list is empty ###
    if {[llength $domain_list] == 0} {
        puts "ERROR: No domains are configured on the switch"
        return 0
    }

    if {[lsearch $domain_list $domain] != -1 } {
        return 1
    } else {
       puts "ERROR: Domain name \"$domain\" is not valid"
       return 0
    }
}

###############################################################################
#    Description: Matches a mpid against the mep list that is obtained from the
#                 CCDB or ERROR DB whichever has the entry and returns 1 if the
#                 mep id matches otherwise returns 0
#
#    IN:  mep id, domain name, ma name, cmd history and policy flags
#    OUT: Returns 1 if there is a match.  Otherwise, returns 0
#
###############################################################################

proc validate_mpid {mpid cmd domain policy ma} {

    global trace
    set mlist {}

    if {$mpid == "NULL"} {
        return 1
    }

    get_remote_mepids $cmd $domain $policy $ma mlist
    debug $trace "mep list: $mlist" 2
    ### get mpids from ccdb, if its empty get the mpids from the error db ###
    if {[llength $mlist] != 0} {
        if {[lsearch $mlist $mpid] != -1} {
            return 1
        } else {
            puts "ERROR: MEP-ID \"$mpid\" is not valid"
            return 0
        }
    } else {
        set mlist [examine_errdb $policy "" $domain $ma $mpid $cmd treason\
                   oreason report maclist]
        debug $trace "mep list: $mlist" 2
        if {[llength $mlist] != 0} {
            if {[lsearch $mlist $mpid] != -1} {
                return 1
            } else {
                puts "ERROR: MEP-ID \"$mpid\" is not valid"
                return 0
            }
        } else {
            puts "ERROR: There is no MEP-ID in the CCDB or ERROR DB"
            return 0
        }
    }
}

###############################################################################
#
#   Description: Procedure to parse domain level from the output of the show
#                command "show ethernet cfm domain <domain name>"
#
#   IN: domain name and command variable to update the command history
#   OUT: Returns the domain level and updates the command history after
#        parsing the domain db
#
###############################################################################

proc parse_domain_level { domain cmd2 policy level2} {

    global trace
    debug $trace "Inside parse_domain_level"
    upvar $cmd2 cmd
    upvar $level2 level
    set show_cmd "show ethernet cfm domain $domain"
    if {$policy == 1} {
        set result [ios_enable_command2 $show_cmd cmd ]
    } else {
        if {[catch {exec "$show_cmd" } result] != 0} {
            puts "ERROR: $show_cmd did not execute properly"
            return 0 
        }
    }
    set level ""
    debug $trace "Domain output: $result" 3
    if { $result != "" } {
        if { [regexp "Invalid input detected" $result a] == 1 } {
            puts "ERROR: INPUT IS NOT VALID\n$result" 
            return 0 
        } else {
            foreach x [split $result "\n"] {
                if { [regexp "Level:" $x config] == 1} {
                    set level [parse_line $x]
                }
            }
        }
    } else {
        puts "Result is null:$result"
        return 0
    }
    debug $trace "Level: $level" 2
    return 1 
}

###############################################################################
#
#   Description: Procedure to parse the line that is passed to it as argument
#                and return the value after the ":" in the line
#
#   IN: Line that is to be parsed
#   OUT: Returns the value after the : in the line after truncating any
#        additional white spaces
#
###############################################################################

proc parse_line {line} {

    global trace
    debug $trace "Inside parse_line....."
    if {$line != ""} {
        set value [split $line ":"]
        set result [lindex $value 1]
        regsub -all {[ \t\r\n]+} $result " " temp
        string trim $temp
    }

    debug $trace "$temp" 2
    return $temp
}

###############################################################################
#
#   Description: Procedure to parse the ccdb output and return details like
#                interface status, port status, MAC address, RDI status and
#                CC error packet statistics
#
#   IN: reference variables for the interface state, port state, rdi state,
#       cc error packet statistics, mac address, ios command, transient error
#       and variables that are passed by value like domain name, vlan-id and
#       mpid
#   OUT: Updates the required ccdb details in the respective reference
#        variables after parsing the ccdb
#
###############################################################################

proc examine_ccdb { intf_state port_state  rdi cc_err_pkt mac mpid domain\
                    vlan transient_err cmd2 } {

    global trace
    upvar $intf_state intf_status
    upvar $port_state port_status
    upvar $rdi RDI
    upvar $cc_err_pkt CC_PKT
    upvar $transient_err transient
    upvar $cmd2 cmd
    upvar $mac mac_addr
    set mac_addr ""
    set intf_status ""
    set port_status ""
    set CC_PKT ""
    set RDI ""

    debug $trace "Inside examine_ccdb....."
    set show_cmd "show ethernet cfm maintenance-points remote detail mpid\
                  $mpid domain $domain vlan $vlan"
    debug $trace "Show command : $show_cmd" 3
    set output [ios_enable_command2 $show_cmd cmd]
    if {[string first "Invalid input detected at '^' marker" $output] != -1 } {
        puts "\n\nERROR: INPUT IS NOT VALID\n$output"
        return 0
    } elseif {$output != ""} {
        set result [split $output \n]
        debug $trace "CCDB output: $output" 3
        ### parse the ccdb output to collect all the details ###
        foreach x $result {
            switch -regexp -- $x {

                "MAC Address:" {
                    set temp [parse_line $x]
                    regsub -all {[ \t\r\n]+} $temp "" mac_addr
                }
                "MEP interface status:" {
                    set temp [parse_line $x]
                    regsub -all {[ \t\r\n]+} $temp "" intf_status
                }
                "MEP port status:" {
                    set temp [parse_line $x]
                    regsub -all {[ \t\r\n]+} $temp "" port_status
                }
                "Receive RDI:" {
                    set temp [parse_line $x]
                    regsub -all {[ \t\r\n]+} $temp "" RDI
                }
                "CC Packet Statistics:" {
                    set temp [parse_line $x]
                    regsub -all {[ \t\r\n]+} $temp "" CC_PKT
                }
                default {
                    # do nothing
                }
            }
            # end of switch
        }
        # end of for
    } else break

    debug $trace "cc_pkt from ccdb:$CC_PKT, RDI from ccdb:$RDI, Port status\
                  from ccdb:$port_status, Interface status from\
                  ccdb:$intf_status, MAC from ccdb:$mac_addr" 3 

    if {($mac_addr != "") && ($intf_status != "") && ($port_status != "")\
        && ($RDI != "") && ($CC_PKT != "")} {
        ### If there is output then it may be transient error ###
        set transient "true"
        return 1
    } else {
        ### If there is no output then it may be timeout error ###
        set transient "false"
        return 1
    }
    debug $trace "Transient Error: $transient" 2
}

###############################################################################
#
#   Description: Procedure to parse the domain db and return the vlan-id and
#                level number if the ma name that is passed to the procedure
#                matches the ma name in the domain db
#
#   IN: domain name, ma name and policy flag are passed by values; ios command
#       vlan-id and domain level are passed by reference variables
#   OUT: Updates the vlan-id, domain level and ios command history reference
#        variables after parsing the domain db
#
###############################################################################

proc get_domain_details {domain MA policy cmd vlan level} {

    global trace
    debug $trace "Inside get_domain_details...."
    set ios_command "show ethernet cfm domain $domain"
    set keyList {Type Crosscheck MA-Name}
    upvar $cmd cmd2
    upvar $vlan vlan2
    upvar $level level2
    set match 0
    set tokenList [get_row_data_by_keys $ios_command $keyList $policy cmd2]

    set size 9
    set maIndex 8
    set vlanIndex 1
    set len [llength $tokenList]
    debug $trace "# of tokens: $len" 3
    for { set i 0 } { $i < $len } { incr i $size } {
        set ma [lindex $tokenList [expr $i + $maIndex]]
        if { [string match $ma $MA] } {
            set match 1
            debug $trace "Matches: $ma $MA" 3
            set vlan2 [lindex $tokenList [expr $i + $vlanIndex]]
            debug $trace  "VLAN :$vlan2" 3
        }
    }
    
    if {$match == 1} {
        if {[parse_domain_level $domain cmd2 $policy level2] == 1} {
            debug $trace "$level2 is level inside get_domain_details, $vlan2\
                          is vlan, $ma is MA" 2
        } else {
            return 0
        }
    } else {
        return 0
    }
    return 1
}

###############################################################################
#
#   Description: Procedure to get the remote mpids associated with the MA name
#                that is passed to it
#
#   IN: domain name, ma name and policy flag are passed by values;
#       command history and meplist variables are passed by reference
#   OUT: Updates the ios command history and meplist reference variables
#        after parsing the remote mep details
#
###############################################################################

proc get_remote_mepids {cmd domain policy MA meplist} {

    global trace
    debug $trace "Inside get_remote_mepids...."
    upvar $cmd cmd2
    upvar $meplist meplist2
    set meplist2 {}
    set tokenList {}
    set ios_cmd "show ethernet cfm maintenance-points remote domain $domain"
    debug $trace $ios_cmd 3
    set tokenList [get_row_data $ios_cmd $policy cmd2]

    set len [llength $tokenList]
    set columns 15
    set mpidIndex 0
    set maIndex 9

    ### Iterate each row of the table output to get the mpid if ma matches ###
    for { set i 0 } { $i < $len } { incr i $columns } {
        debug $trace "[lindex $tokenList $i]" 3
        set ma [lindex $tokenList [expr $i + $maIndex]]
        debug $trace "MA from ccdb: $ma after" 3
        debug $trace "MA from input: $MA after" 3
        if {[string match $ma $MA]} {
            lappend meplist2 [lindex $tokenList  [expr $i + $mpidIndex]]
        }
    }
    debug $trace "meplist: $meplist2" 2
}

###############################################################################
#
#   Description: Procedure to parse error reason from a list of row data where
#                each item in the list is inturn a list of data related to a
#                MEP
#
#   IN: list that contains items corresponding to a MEP, mepid are passed by
#       values; mepid, mac address and error reason variables are passed by
#       reference
#   OUT: Updates mepid, mac address and error reason reference variables after
#        parsing the list of data passed to it
#
###############################################################################

proc get_error_reason {row_data_list mac_addr reason mepid} {

    global trace
    debug $trace "In get_error_reason......"
    upvar $reason err_reason
    upvar $mac_addr mac
    upvar $mepid mpid
    set err_reason "-"
    set len [expr [llength $row_data_list] - 1]

    debug $trace "In get_error_reason List is: $row_data_list" 3
    set mac [lindex $row_data_list 0]
    string trim $mac
    debug $trace "Length in get_error_reason: $len" 2
    set lastindex [expr $len - 1]
    for {set i 0} {$i < $len} {incr i} {
        if { ($i > 4) && ($i < $lastindex)} {
            if {$i == [expr $lastindex - 1]} {
                append err "[lindex $row_data_list $i]"
            } else {
                append err "[lindex $row_data_list $i] "
            }
        }
    }
    set err_reason $err
    debug $trace "In get_error_reason Reason is :$err_reason" 2
    debug $trace "In get_error_reason MAC is :$mac" 3
    set mpid [lindex $row_data_list $len]
    debug $trace "other row's  mep id is :$mpid" 2
}

###############################################################################
#
#   Description: Procedure to prepare error db report
#
#   IN: scenario type, mpid from errdb, mpid given as input, list which\
#       contains one row of data, last (flag which indicates last row data)\
#       are passed by values; mac list and error reason variables are passed by
#       reference
#   OUT: Updates error reason and error report reference variables with the
#         reason and report
#
###############################################################################

proc prepare_errdb_report { scenario db_mepid mpid temp_list last mac_list\
     t_reason o_reason err_report id } {

    global trace
    upvar $mac_list mlist
    upvar $t_reason timeout_reason
    upvar $o_reason other_reason
    upvar $err_report report
    upvar $id new_mpid
    set mac ""
    
    debug $trace "Inside prepare_errdb_report......"
    string trim $db_mepid
    debug $trace "db mepid: $db_mepid, mpid: $mpid" 2
    if {($db_mepid == $mpid) || ($db_mepid == "-") || ($mpid == "NULL")} {
        get_error_reason $temp_list mac reason id
        if {($last) || ($db_mepid == "-") || ($mpid == "NULL")} {
            append report "    (a) MEP ID - $db_mepid\n"
        } elseif {$id == $mpid} {
            append report "    (a) MEP ID - $id\n"
        } else {
            append report "    (a) MEP ID - $mpid\n"
        }
        append report "    (b) MAC Address - $mac\n"
        append report "    (c) Reason - \"$reason\"\n\n"
    } else {
        get_error_reason $temp_list mac reason id
    }
    set new_mpid $id
    debug $trace "In prepare_errdb_report - Reason: $reason" 3
    ### if timeout is the reason then prepare a list of the MACs ###
    if {$db_mepid == $mpid} {
        if {[regexp "Lifetime Timer Expired" $reason] == 1} {
            append timeout_reason "$reason, "
            lappend mlist "$mac:$reason"
        } else {
            append other_reason "$reason, "
        }
    } elseif {$db_mepid == "-"} {
        append other_reason "$reason, "
    } elseif {$mpid == "NULL"} {
        if {[regexp "Lifetime Timer Expired" $reason] == 1} {
            append timeout_reason "$reason, "
            lappend mlist "$mac:$reason"
        } else {
            append other_reason "$reason, "
        }
    } else {
        # do nothing
    }
}

###############################################################################
#
#   Description: Procedure to examine the error db and get the error reasons
#
#   IN: scenario type, domain, mpid and ma name are passed by values; command
#       history, mac address, mac list, errordb report and error reason 
#       variables are passed by reference
#   OUT: Updates the ios command history and error reason reference variables
#        after parsing the error db
#
###############################################################################

proc examine_errdb { policy scenario domain ma mpid cmd t_reason o_reason\
                     err_report mac_list} {

    global trace
    debug $trace "Inside examine_errdb....."
    upvar $cmd cmd2
    upvar $t_reason timeout_reason
    upvar $o_reason other_reason
    upvar $err_report report
    upvar $mac_list mlist
    set timeout_reason "-"
    set other_reason ""
    set temp_list {}
    set mlist {} 
    set meplist {}
    set hostname [info hostname]

    set ios_cmd "show ethernet cfm errors domain-id $domain service $ma"
    set tokenList [get_row_data $ios_cmd $policy cmd2]

    set mepid [lindex $tokenList 0]
    lappend meplist $mepid
    string trim $mepid
    set domain_name [lindex $tokenList 1]
    string trim $domain_name
    debug $trace "Domain is :$domain_name, Mpid is :$mepid" 3
    for {set y 2} {$y < [llength $tokenList]} {incr y} {
        set value [lindex $tokenList $y]
        if {$value != $domain} {
            lappend temp_list $value
            ### To handle the last row of the table output ###
            if {[string compare -length 3 $value $hostname] == 0} {
                prepare_errdb_report $scenario $mepid $mpid $temp_list 1\
                    mlist timeout_reason other_reason report id 
                lappend meplist $id
            }
        } else {
            ### Take each row of errdb data ###
            prepare_errdb_report $scenario $mepid $mpid $temp_list 0 mlist\
                timeout_reason other_reason report id
            set mepid $id
            lappend meplist $id
            set temp_list {}
        }
    }

    debug $trace "In examine_errdb t_reason: $timeout_reason" 2
    debug $trace "In examine_errdb o_reason: $other_reason" 2
    debug $trace "In examine_errdb mac: $mlist" 2
    if {$policy == 0} {
        return $meplist
    }
}

###############################################################################
#
#   Description: Procedure to do the ping and record the success or failure
#                of the ping
#
#   IN: mac address, domain name and vlan-id variables are passed by values;
#       command history, ping status and ping reason variables are passed by
#       reference
#   OUT: Updates the command history, ping status, ping report and ping reason
#        reference variables based on the success or failure of the ping
#
###############################################################################

proc cfm_ping { mac domain vlan cmd ping_result ping_report} {

    global trace
    debug $trace "Inside cfm_ping......"
    upvar $cmd cmd2
    upvar $ping_result result
    upvar $ping_report report
    set reason ""
    set output ""

    set p_cmd "ping ethernet $mac domain $domain vlan $vlan"
    debug $trace "Ping command: $p_cmd" 3
    if {[string compare "" $mac] == 0 || [string compare "" $domain] == 0\
        || [string compare "" $vlan] == 0 } {
         puts "ERROR: Ping command \"$p_cmd\" cannot be executed"  
        return 0
    } else {
        set output [ ios_enable_command2 "ping ethernet $mac domain $domain\
                     vlan $vlan" cmd2]
    }    

    if {[string first "Invalid input detected at '^' marker" $output] != -1 } {
        puts "INPUT IS NOT VALID\n$output"
        return 0
    }
    if { $output != "" } {
        if { [regexp "Success rate is 0 percent" $output] == 1 } {
            set result "fail"
            append report "    Ping to \"$mac\" failed.\n" 
        } else {
            set result "pass"
            append report "    Ping to \"$mac\" was successful.\n"
        }
    }
    return 1
    debug $trace $report 2 
}

###############################################################################
#
#   Description: Procedure to do the traceroute and record the success or\
#                failure of the traceroute 
#
#   IN: mac address, domain name and vlan-id variables are passed by values;
#       command history,  trace result and trace output variables are passed by
#       reference
#   OUT: Updates the command history, trace status, trace output by 
#        reference variables based on the success or failure of the trace
#        output 
#
###############################################################################

proc is_trace_success { mac domain vlan cmd trace_result trace_output} {

    global trace "Inside is_trace_success......." 
    upvar $cmd cmd2
    upvar $trace_result result
    upvar $trace_output output
    set ios_cmd "traceroute ethernet $mac domain $domain vlan $vlan"
    set output [ ios_enable_command2 $ios_cmd cmd2]

    if { $output != "" } {
        if { [string first "Invalid input detected at '^' marker" $output]\
              != -1 } {
            puts "INPUT IS NOT VALID\n$output"
            exit 
        } elseif { [regexp "RlyHit:MEP" $output] == 1 } {
            set result 1
        } else {
            set result 0
        }
    }
}

###############################################################################
#
#   Description: Procedure to return error code corresponding to the list of
#                errors passed.
#
#   IN: errorList 
#   OUT: Returns error code list
#
###############################################################################

proc get_error_code {errList} {

    global trace
    debug $trace "Inside get_error_code......."
    upvar $errList errcodeList
    set errcodeList {}

    if {[llength $errList] != 0} { 
        foreach x $errList {
            switch -regexp -- $x {
                "Lifetime Timer Expired" {
                    lappend errcodeList "Timeout"               
                }
                "Crossconnect Error" {
                    lappend errcodeList "Xconn"               
                }
                "Loop Trap Error" {
                    lappend errcodeList "Loop"               
                }
                "Configuration Error" {
                    lappend errcodeList "Config" 
                }
                "Remote MEP missing" {
                    lappend errcodeList "MissMEP"               
                }
                "Unknown MEP" {
                    lappend errcodeList "UknMEP"               
                }
                "Receive RDI" {
                    lappend errcodeList "RxRDI"               
                }  
                "Receive LCK" {
                    lappend errcodeList "RxLCK"               
                }  
                "Receive AIS" {
                    lappend errcodeList "RxAIS"               
                }  
                default {
                    # do nothing
                }
            }
        } 
    }

    debug $trace "Error Codes: $errcodeList" 2
    return $errcodeList    
}

###############################################################################
#
#   Description: Procedure to do generate summary and detailed report for
#                timeout and ondemand policies after calling different
#                procedure that examines ccdb, errordb, does ping and
#                traceroute for the given mep and ma
#
#   IN: domain name, vlan-id, ma name, mepid, domain level and scenario type
#       variables are passed by values; command history, summary and detail
#       variables are passed by reference
#   OUT: Updates the command history, summary and details reference
#        variables
#
###############################################################################

proc prepare_report {cmd sum2 detail2 mpid domain vlan level ma scenario} {

    global trace
    debug $trace "Inside prepare_report......"
    upvar $cmd command
    upvar $sum2 sum
    upvar $detail2 detail
    set intf_state ""
    set port_state ""
    set RDI ""
    set CC_PKT ""
    set mac ""
    set errList {}
    set errdb_entry 0
    set errdb_o_entry 0
    set policy 1
    set report ""

    set msg " Diagnostics for rmep " 
    if {$scenario == "timeout"} {
        append sum "SERV-DIAG_CFM_timeout:" 
        append detail "DETAILED MESSAGE:\n\n" 
    } else {
        append sum "SERV-DIAG_CFM_ondemand:" 
        append detail "DETAILED MESSAGE:\n\n"
    }
    append sum $msg
    append detail $msg

    ### Examine ccdb ###
    if {[examine_ccdb intf_state port_state RDI CC_PKT cc_mac $mpid $domain\
                      $vlan transient command] == 0} {
        return 0
    }

    ### Examine error db ###
    examine_errdb $policy $scenario $domain $ma $mpid command reason o_reason\
                  report mac_list 

    ### Get count of Error DB entry and list of error codes ###
    get_count_of_errors $o_reason $mac_list errdb_entry errList
    if {[llength $mac_list] == 0} {
        set mac $cc_mac
    } else {
        set mac [lindex [split [lindex $mac_list 0] ":"] 0]
    }

    ### do ping and traceroute ###
    if {[cfm_ping $mac $domain $vlan command ping p_report] == 0} {
        return 0
    } 
    get_trace_info $mac $domain $vlan command t_status t_output cache_output\
                   $scenario $level

    ### Append summary and details ###
    append sum $mac
    append detail $mac
    set msg " on vlan $vlan, CFM MD"
    set short_domain [get_trimmed_domain $domain]
    append sum "$msg $short_domain."
    append detail "$msg $domain"
    if {$level != ""} {
        set msg ", level = $level; "
    }
    append detail "$msg\n\n"

    ### Remove last comma in the other reason string ###
    set o_reason [string range $o_reason 0\
                  [expr [string length $o_reason] - 3]]

    ### call procedure generate_report with all necessary parameters ###
    generate_report $transient $mpid $mac $intf_state $port_state $RDI $reason\
        $errdb_entry $errList $report $CC_PKT $o_reason $ping $p_report\
        $t_status $t_output $cache_output $scenario summary details
    append sum $summary
    append detail $details
}

###############################################################################
#
#   Description: Procedure to do generate summary and detailed report for
#                timeout and ondemand policies based on the parameters passed 
#                and updates the summary and detail reference variables with 
#                the generated report 
#
#   IN: The variables for transient error, mpid, mac, interface status, port
#       status, RDI, timeout reason, other reasons, cc error packet statistics,\
#       ping status, ping report, ping failure reason, traceroute status,\
#       traceroute report of last hop are passed by values; command history,\
#       summary and detail variables are passed by reference
#   OUT: Updates summary and details reference variables
#
###############################################################################

proc generate_report { transient mpid mac intf_state port_state RDI reason\
     errdb_entry errList report CC_PKT o_reason ping p_report t_status\
     t_output cache_output scenario sum2 detail2 } {

    global trace
    upvar $sum2 sum
    upvar $detail2 detail
    debug $trace "Entering generate_report......"
    ### Append summary and details ###
    if {$transient} {
        append sum "CCDB: 1 found, "
        if {[string match "Up" $intf_state] == 1} {
            append sum "1 with UP IF state; "
        } else {
            append sum "0 with UP IF state; "
        }
        append detail "(1) Details of remote MEP collected from CCDB:\n"
        append detail "    (a) MEP ID - $mpid\n"
        append detail "    (b) MAC Address - $mac\n"
        append detail "    (c) Interface status - $intf_state\n"
        append detail "    (d) Port status - $port_state\n"
        append detail "    (e) RDI state - $RDI\n"
        append detail "    (f) CC packet statistics - $CC_PKT\n\n"
    } else {
        append sum "CCDB: 0 found; "
        append detail "(1) Details of remote MEP was not found in CCDB.\n\n"
    }

    append detail "(2) Details of remote MEP collected from Error DB\n"
    if {$errdb_entry != 0} {
        append sum "ErrorDB: $errdb_entry found; Reasons: "
        set len [llength $errList]
        for {set a 0} {$a < $len} {incr a} {
            if {$a != [expr $len - 1]} {
               append sum "[lindex $errList $a], "
            } else {
               append sum "[lindex $errList $a]; "
            }
        }
        append detail $report
        #append detail "    (a) MEP ID - $mpid\n"
        #append detail "    (b) MAC Address - $mac\n"
        #append detail "    (c) Reason(s) - $reason, $o_reason\n\n"
    } else {
        append sum "ErrorDB: $errdb_entry found; Reasons: none; "
        if {$errdb_entry >= 1} {
            append detail $report
            #append detail "    (a) MEP ID - $mpid\n"
            #append detail "    (b) MAC Address - $mac\n"
            #append detail "    (c) Reason - $o_reason\n\n"
        } else {
            append detail "No Error entry was found in Error DB\n\n" 
        }
    }

    if { $ping == "pass" } {
        append sum "Ping: pass; "
        append detail "(3) Output of loopback operation performed to\
               remote MEP: $p_report."
        if {($transient) && ($t_status) && ($scenario == "timeout")} {
            append detail "TRANSIENT ERROR\n\n"
        } elseif {$scenario == "timeout"} { 
            append detail "A multicast delivery problem could exist in the\
                   network (that prevents CCMs to arrive) or the remote MEP\
                   has CC protocol disabled.\n\n"
        } else {
            # do nothing as it is ondemand
        }
    } else {
        append sum "Ping: fail; "
        append detail "(3) Output of loopback operation performed to remote\
               MEP : $p_report\n"
    }
    if { $t_status } {
        append sum "Trace: pass"
        append detail "(4) Output from Traceroute operation to remote MEP :\
               Traceroute successful.\n\n"
        append detail "(5) TRACEROUTE OUTPUT:$t_output"
    } else { 
        append sum "Trace: fail"
        append detail "\n(4) Output from Traceroute operation to remote MEP :\
               Traceroute failed.\n\n"
        append detail "(5) TRACEROUTE OUTPUT:$t_output"
        if {$scenario == "demand"} {
            append detail "\n(6) $cache_output"
        }
    }

}

###############################################################################
#
#   Description: Procedure to do multicast ping and get the count and mac 
#                address of those MEPs that replied  
#
#   IN: domain name and vlan-id are passed by values; command history is passed
#       by reference
#   OUT: Updates the command history and a mac list that contains a list of 
#        MAC address of MEPs that replied to multicast ping
#
###############################################################################

proc multicast_ping { cmd  mac_list domain vlan } {

    global trace
    upvar $cmd cmd2
    upvar $mac_list mlist
    
    debug $trace "Inside multicast_ping....."
    set ios_cmd "ping ethernet multicast domain $domain vlan $vlan"
    set output [ios_enable_command2 $ios_cmd cmd2]
    debug $trace "Multicast ping: $output" 3
    set result [split $output "\n"]
    debug $trace "Multicast ping: $result" 3
    foreach x $result {
        if { [regexp "Reply to Multicast request from " $x a ] == 1 } {
            regsub -all {[ \t\r\n]+} $x " " temp
            set temp [split $temp " "]
            debug $trace "List: $temp" 3
            set mac [lindex $temp 5]
            set temp [split $mac ","]
            set mac [lindex $temp 0]
            debug $trace "MAC is: $mac" 3
            lappend mlist $mac
        }
    }
    debug $trace "Inside multicast_ping mac list: $mlist"
}

###############################################################################
#
#   Description: Procedure to compare macs from ccdb and macs that responded to
#                multicast ping 
#
#   IN: list of macs from ccdb and list of macs from multicast macs are passed
#       by values and mdiff is passed by reference
#   OUT:  Updates mdiff the difference between macs from multicast ping and 
#         macs from ccdb by reference
#
###############################################################################

proc compare_ccdb_and_multicast_mac { mlist_multicast mlist_ccdb mdiff } {

    global trace
    debug $trace "Inside compare_ccdb_and_multicast_mac......"
    upvar $mdiff diff
    set match_mac_list {}
    set count 0

    if {[llength $mlist_ccdb] == 0} {
        append diff "Since no entry was found in CCDB. A multicast delivery\
                     problem could exist in the network.\n"
    }

    if {[llength $mlist_multicast] == 0} {
        append diff "Remote MEPs could have CC protocol disabled"
    }

    append diff "The MAC address/addresses successful from multicast ping and\
           not found in CCDB are: "
    
    foreach mac $mlist_multicast {
        set match 0
        foreach mac_ccdb $mlist_ccdb {
            regsub -all {[ \t\r\n]+} $mac "" mac
            regsub -all {[ \t\r\n]+} $mac_ccdb "" mac_ccdb
            if {[string compare $mac $mac_ccdb] == 0} {
                set match 1
                break
            }
        }
        if {$match != 1} {
            append diff "\"$mac\", "
            incr count
        } else {
            lappend match_mac_list $mac
        }
    }
    if { $count == 0 } {
        append diff "\tNIL"
    } else {
        append diff "\nSome Remote MEPs could have CC protocol disabled."
    }
    if {[regexp "NIL" $diff] == 1} {
        set diff ""
    }
   
    debug $trace $diff 2
}

###############################################################################
#
#   Description: Procedure to examine the traceroute cache and return the\
#                output of successful traceroute if present in the cache. 
#                Otherwise returns empty string or message indication if the\
#                traceroute cache is enabled or not
#
#   IN: mac address is passed by value; command history and trace-report are
#       passed by reference
#   OUT: Updates the command history and trace report with the warning message
#        if cache is not enabled or successful traceroute information for the
#        mac that was passed to it
#
###############################################################################

proc examine_cache { cmd trace_report mac domain vlan level} {

    global trace
    upvar $cmd cmd2
    upvar $trace_report s_trace
    debug $trace "Inside examine_cache......"
    set output ""
    set templist {}
    set s_trace "\n"
    set msg "" 
    set policy 1
    regsub -all {[ \t\r\n]+} $level "" y
    set level $y

    ### check if traceroute cache is enabled ###
    set result [is_traceroute_cache_enabled $policy cmd2 output]
    if {$result == 0} {
        set s_trace "Traceroute Cache is not Enabled. Please enable traceroute\
            cache."
        return 
    }
    
    if {[string first "Current Cache-size: 0 Hops" $output] != -1} {
        set s_trace "Traceroute Cache is empty. Please define cache hold time\
                     and cache size appropriately."
        return 
    }
    debug $trace "Tracerout-cache:$output" 3
    foreach x [split $output "\n"] {
        lappend templist $x
    }
    debug $trace "Cache Entry list:$templist" 3
    set len [llength $templist]
    debug $trace "List length:$len" 3
    set startIndex [expr $len - 2]
    debug $trace "Start index:$startIndex" 3
    for {set a $startIndex} {$a >= 0} { } {
        set value [lindex $templist $a] 
        debug $trace $value
        if {[regexp "Traceroute to $mac on Domain $domain, Level $level, vlan\
                     $vlan issued at" $value] == 1} {
            set s_trace "\n(5) Previous successful trace route from cache is:\n"
            for {set i $a} 1 {incr i} {
                set value2 [lindex $templist $i]
                append s_trace $value2
                if {[regexp "RlyHit:MEP" $value2] == 1} {
                    break
                }
                if {($i == $startIndex) || ([regexp "Traceroute to"\
                    [lindex $templist [expr $i+1] ] ] == 1) } {
                    debug $trace $i 3
                    debug $trace $value2
                    set s_trace ""
                    break
                }
            }
            # end of inner for loop
            if {$s_trace != ""} {
                debug $trace "Success trace from cache: $s_trace" 2
                break
            }
        }
        # end of if condition that matches the traceroute mac
        set a [expr $a-1]
    }
    # end of outer for loop
    if {$s_trace == ""} {
        set s_trace "No Successful traceroute entry was found in the cache\n"
    }
    debug $trace "$s_trace is output from examine_cache" 
}

###############################################################################
#
#   Description: Procedure to prepare ccdb report based on details found in the
#                ccdb that is passed to it and updates the report through a
#                reference variable
#
#   IN: mep id, mac address, interface state, port state, RDI error, cc packet
#       statistics are passed by values and ccdb report variable is passed by
#       reference
#   OUT: Updates the ccdb report varaible with the prepared report
#
###############################################################################

proc prepare_ccdb_report { mpid mac intf port rdi ccpkt cc_report} {

    global trace
    upvar $cc_report report
    debug $trace "Inside prepare_ccdb_report......"
    append report "    (a) MEP ID - $mpid\n"
    append report "    (b) MAC Address - $mac\n"
    append report "    (c) Interface status - $intf\n"
    append report "    (d) Port status - $port\n"
    append report "    (e) RDI state - $rdi\n"
    append report "    (f) CC packet statistics - $ccpkt\n"
    append report \n

    debug $trace "CCDB Report:\n$report"

}

###############################################################################
#
#   Description: Procedure to get the number of error entries in errordb 
#
#   IN: Error reasons, mac list and timeout error reason combination are
#       passed by values and count is passed by reference 
#   OUT: Updates the error entries in the count variable and returns the
#        mac addresses in the error database
#
###############################################################################

proc get_count_of_errors {o_reason mac_list count errList} {

    global trace
    debug $trace "Inside get_count_of_errors......" 
    upvar $count cnt
    upvar $errList errorList
    set errorList {}
    set mlist {}
    set cnt 0

    ### Get a count of other error reasons ###
    set temp [split $o_reason ","]
    if {[llength $temp] != 0} {
        foreach x $temp {
            lappend errorList $x
            regsub -all {[ \t\r\n]+} $x "" y
            if {$y != ""} {
                incr cnt
            }
        }        
    }
    
    ### Get a count of timeout errors ###
    if {[llength $mac_list] != 0} {
        foreach x $mac_list {
            set temp [split $x ":"]
            lappend errorList [lindex $temp 1]
            set reason [lindex $temp 1]
            lappend mlist [lindex $temp 0]
            regsub -all {[ \t\r\n]+} $reason " " y
            debug $trace "Reason is: $y"
            if {$y != "-"} {
                incr cnt
            }
        }
    }
    
    set errorList [get_error_code $errorList]
    set errorList [lsort -unique $errorList]
    debug $trace "Error DB entry:$cnt"
    debug $trace "Mac address from errordb with timeout error: $mlist"
    return $mlist
}

###############################################################################
#
#   Description: Procedure to get the number of meps and the number of meps
#                with interface state up 
#
#   IN: intf state is passed by value, intf_up and count variables
#       are passed by reference 
#   OUT: Updates the interface up counts and mep counts in the 
#        variables passed by reference 
#
###############################################################################

proc get_mep_counts { intf_state intf_up count} {

    global trace
    debug $trace "Inside get_mep_counts......"
    upvar $intf_up up
    upvar $count cnt

    incr cnt
    regsub -all {[ \t\r\n]+} $intf_state "" intf 
    if {[string match $intf "Up"]} {
        incr up
    }
    debug $trace "Mep Count: $cnt" 2
    debug $trace "Interface Up Count: $up" 2
}

###############################################################################
#
#   Description: Procedure to get ping results specific to ondemand scenario
#                and get a count of passed and failed ping counts for both
#                mac addresses from the ccdb and mac addresses from error db
#                that has the "Lifetime Timer Expired" error
#
#   IN: list of mac addresses, domain, vlan-id are passed by values and command
#       history, ping result, ping report, ping passed and failed counts are
#       passed by references 
#   OUT: Updates the ping passed, ping failed, ping result, ping report and 
#        command history variables 
#
###############################################################################

proc ping_for_ondemand { mlist domain vlan cmd p_result p_report p_cnt\
                         p_fail_cnt } {
    global trace
    upvar $cmd cmd2
    upvar $p_result ping
    upvar $p_report report
    upvar $p_cnt ping_cnt
    upvar $p_fail_cnt ping_fail_cnt

    if {[llength $mlist] != 0} {
        foreach a $mlist {
            if {[cfm_ping $a $domain $vlan cmd2 ping report] == 0} {
                return 0
            }
            if {$ping == "pass"} {
                incr ping_cnt
            } else {
                incr ping_fail_cnt
            }
        }
    } else {
        set ping ""
        append report ""
    }
    return 1
}

###############################################################################
#
#   Description: Procedure to do traceroute and get the output of successful
#                traceroute from the traceroute cache if the issued traceroute\
#                was not successful 
#
#   IN: list of mac addresses, domain, vlan-id are passed by values and command
#       history, trace result, trace report, traceroute cache output are 
#       passed by references
#   OUT: Updates the trace status, trace report, successful trace from cache
#        and command history variables
#
###############################################################################

proc get_trace_info {mlist domain vlan cmd t_status t_report c_output\
                     scenario level} {

    global trace
    debug $trace "Inside get_trace_info......"
    upvar $cmd cmd2
    upvar $t_status status
    upvar $t_report report
    upvar $c_output cache_output
    set cache_output ""
    
    if {[llength $mlist] != 0} {
        foreach i $mlist {
            is_trace_success $i $domain $vlan cmd2 status report
            if {($status == 0) && ([string match "demand" $scenario] == 1)} {
                examine_cache cmd2 cache_output $i $domain $vlan $level
            }
        }
    } else {
        set report "No MEPs with \"Lifetime Timer Expired\" reason is found\
            in Error DB. Hence no traceroute operation is peformed."
        set status "-1" 
    }
}

###############################################################################
#
#   Description: Procedure to collect information from ccdb if there is any 
#                entry for a list of meps and prepare a ccdb report 
#
#   IN: list of meps, domain, vlan-id are passed by values and command
#       history, ccdb report, mac list, mep count, interface up count 
#       and transient flag are passed by references
#   OUT: Updates  ccdb report, mac list, mep count, interface up count 
#        and transient flag and command history variables
#
###############################################################################

proc get_ccdb_info {meplist domain vlan ccdb_report mlist mep_count int_up\
                    transient} {

    global trace
    upvar $ccdb_report report
    upvar $mlist mac_list
    upvar $mep_count mep_cnt
    upvar $int_up intf_up
    upvar $transient transient_err
    upvar $mlist mac_list
    set transient_err "false"
    set mac_list {}
    set mep_cnt 0
    set intf_up 0

    set len [llength $meplist]
    debug $trace "meplist: $meplist" 3
    if {$len != 0} {
        append report "(1) Details of remote MEPs collected from CCDB:\n"
        foreach x $meplist {
            ### Examine ccdb ###
            if {[examine_ccdb intf_state port_state RDI CC_PKT mac $x $domain\
                              $vlan transient_err cmd2] == 0} {
                return 0
            }
            if {$transient_err} {
                ### update mep count, interface up state counts ###
                get_mep_counts $intf_state intf_up mep_cnt

                ### prepare ccdb report ###
                prepare_ccdb_report $x $mac $intf_state $port_state $RDI\
                    $CC_PKT report

                ### get a list of mac addresses which have entry in the ccdb ###
                lappend mac_list $mac
            }
        }
    } else {
        set mep_cnt 0
        set intf_up 0
    }
    if { $report == "" } {
        set report "(1) No CCDB entry was found for the remote MEPs.\n" 
    }
    debug $trace "MAC addresses from ccdb:$mac_list" 2
}

###############################################################################
#
#   Description: Procedure to run all the show commands to do cfm diagnoses
#                and get the information from the show command outputs to
#                prepare  and generate a report based on the diagnostic result
#   IN: domain, vlan-id, ma, level and scenario type are passed by values and
#       command  history, summary and detail report are passed by references
#   OUT: Updates command history, summary and detail report 
#
###############################################################################

proc prepare_ondemand_report {cmd sum detail domain ma vlan level scenario} {

    global trace
    debug $trace "Inside prepare_ondemand_report......."
    upvar $cmd cmd2
    upvar $sum sum2
    upvar $detail detail2
    set  report ""
    set err_report ""
    set reason ""
    set o_reason ""
    set mac_list {}
    set mac_list2 {}
    set mac_list3 {}
    set errList {}
    set policy 1
    set trace_report ""
    set ping_cnt 0
    set ping_fail_cnt 0

    ### Get the MEP ids associated with the ma given by the user ###
    get_remote_mepids cmd2 $domain $policy $ma meplist

    ### Examine ccdb, collect details and prepare report ###
    if {[get_ccdb_info $meplist  $domain $vlan report mac_list mep_cnt intf_up\
                       transient] == 0} {
        return 0
    }

    ### Examine error db ###
    examine_errdb $policy $scenario $domain $ma NULL cmd2 reason o_reason\
                  err_report mac_list2

    ### update errdb entry count and get a list of mac addresses of those ###
    ### which have the timeout error ###
    set mac_list2 [get_count_of_errors $o_reason $mac_list2 errdb_entry errList]

    ### prepare error db report ###
    append report "(2) Details of remote MEP collected from Error DB\n"
    append report "$err_report"

    ### do fault and connectivity verification ie ping ###
    ### do ping for each mep from ccdb ###
    if {[ping_for_ondemand $mac_list $domain $vlan cmd2 ping p_report ping_cnt\
                           ping_fail_cnt] == 0} {
        return 0
    }
     ### do ping for each mep in errdb with timeout error ###
    if {[ping_for_ondemand $mac_list2 $domain $vlan cmd2 ping p_report ping_cnt\
                           ping_fail_cnt] == 0} {
        return 0
    }

    ### Since transient variable is set for each mepid in ccdb it can be 
    ### overwritten, the following code takes care of it
    if {$ping_cnt > 0} {
        set transient "true"
    }

    ### do multicast ping ###
    multicast_ping cmd2 mac_list3 $domain $vlan

    ### compare ccdb mac address with mac addresses returned from multicast ###
    ### ping ###
    compare_ccdb_and_multicast_mac $mac_list3 $mac_list mac_diff
    ### do fault isolation ie traceroute and get the last hop information ###
    get_trace_info $mac_list2 $domain $vlan cmd2 t_status trace_report\
        cache_output $scenario $level

    ### generate report ###
    generate_ondemand_report cmd2 sum2 detail2 $ma $vlan $domain $level\
         $mep_cnt $intf_up $ping_cnt $ping_fail_cnt $errdb_entry $reason\
         $o_reason $report $p_report $mac_diff $t_status $trace_report\
         $cache_output $errList
}

###############################################################################
#
#   Description: Procedure to generate summary and detail report based on the
#                inputs given to it
#   IN: domain, vlan-id, ma, level, mep count, interface up state count,
#       successful ping count, failed ping count, number of error
#       db entries, error reasons, ccdb and erordb report, ping report, trace
#       report, trace status, trace output, successful cache output and mac 
#       address list  are passed by values and command  history, summary and
#       detail report are passed by references
#   OUT: Updates command history, summary and detail report
#
###############################################################################

proc generate_ondemand_report {cmd sum detail ma vlan domain level mepcnt\
         intf_up ping_cnt ping_fail_cnt errdb_entry reason o_reason\
         report p_report mac_diff t_status trace_report cache_output errList} {

    global trace
    upvar $cmd cmd2
    upvar $sum sum2
    upvar $detail detail2
    set mep_cnt 0

    set msg "SERV-DIAG_CFM_ondemand: "
    append sum2 $msg
    append detail2 "DETAILED MESSAGE:\n"
    set short_domain [get_trimmed_domain $domain]
    append sum2 "Diagnostics for vlan $vlan, CFM MD $short_domain, rmep:all. "
    if {$level != ""} {
        set msg ", level = $level. "
    }
    append detail2 "Diagnostics for vlan $vlan, CFM MD $domain, rmep:all$msg\n"
    append sum2 "CCDB: $mepcnt found, $intf_up with UP IF state; ErrorDB:\
                 $errdb_entry found; Reasons: "
    set len [llength $errList]
    for {set a 0} {$a < $len} {incr a} {
        if {$a != [expr $len - 1]} {
            append sum2 "[lindex $errList $a], "
        } else {
           append sum2 "[lindex $errList $a]; "
        }
    }
    set total [expr $ping_cnt + $ping_fail_cnt]
    append sum2 "Ping: $total/$ping_fail_cnt total/fail;" 
    append detail2 $report
    append detail2 "(3) Output of loopback operation performed:\n"
    append detail2 $p_report
    append detail2 "    Number of successful pings: $ping_cnt\n"
    append detail2 "    Number of failed pings: $ping_fail_cnt\n"
    append detail2 $mac_diff\n
    if {$t_status == -1} {
        append detail2 "(4) $trace_report\n"
    } else {
        append detail2 "(4) Output of traceroute operation to remote MEP: "
        if {$t_status} {
            append detail2 "Successful.\n    TRACEROUTE OUTPUT:$trace_report\n"
        } else {
            append detail2 "Failed.\nTRACEROUTE OUTPUT:$trace_report\n\
                   $cache_output\n"
        }
    }
}


#Cisco Tcl Signature V1.0
#3082077506092a864886f70d010702a082076630820762020101310b3009
#06052b0e03021a0500300b06092a864886f70d010701a08204bb308204b7
#3082039fa003020102020900d48b1bbfb069e9b9300d06092a864886f70d
#0101050500308198310b3009060355040613024341311230100603550408
#130963616c69666f6e69613111300f060355040713086d69706c69746173
#311b3019060355040a1312636973636f2073797374656d7320696e632e31
#0e300c060355040b13054e53535447311630140603550403130d43687269
#7374696e652059616e311d301b06092a864886f70d010901160e6379616e
#40636973636f2e636f6d301e170d3037313231383138333232375a170d31
#30313231373138333232375a308198310b30090603550406130243413112
#30100603550408130963616c69666f6e69613111300f060355040713086d
#69706c69746173311b3019060355040a1312636973636f2073797374656d
#7320696e632e310e300c060355040b13054e535354473116301406035504
#03130d436872697374696e652059616e311d301b06092a864886f70d0109
#01160e6379616e40636973636f2e636f6d30820122300d06092a864886f7
#0d01010105000382010f003082010a02820101009c22f1d01af397756ab6
#cfce92a9e761b73fbf1efd070aa4e90448e6077e2c127ff47a5951de7790
#55da014ff152576a08d7199b8ad5757eb8c9a7c1a493715df2176e9f56f0
#74735d43420cf1ed17fe0aa3494724808aefcf274572c7750be071a94560
#9c045c48a5454f9e7933bf93d967bcf582918b1bbd48f47dd978c7ea9c88
#23eb01482a40ef770f823ea2e63eaf7cab5367bd51b54979925464e27b22
#82fc5bfe94f6f41f2f91e56444ae371a050107fe79fd5db3f228bbbd5fcb
#653a87a3c1268d74500ba1489382290804d9d7937d3269b1c55eae51f9f7
#62eadbd4226ba6d4008414b353181ee6cfbe24204e4a82436b117c255014
#77ec74c3ac4d0203010001a38201003081fd301d0603551d0e041604149c
#0e12d3a222891fc1300b780f38b54e07e228073081cd0603551d230481c5
#3081c280149c0e12d3a222891fc1300b780f38b54e07e22807a1819ea481
#9b308198310b300906035504061302434131123010060355040813096361
#6c69666f6e69613111300f060355040713086d69706c69746173311b3019
#060355040a1312636973636f2073797374656d7320696e632e310e300c06
#0355040b13054e53535447311630140603550403130d436872697374696e
#652059616e311d301b06092a864886f70d010901160e6379616e40636973
#636f2e636f6d820900d48b1bbfb069e9b9300c0603551d13040530030101
#ff300d06092a864886f70d010105050003820101003e6d657e30c21acb30
#d7e91aef99cd6f83a8644d7ae437fb4a0cd1a8c6a0f1cb2194be80868b75
#1d20ee8dc72a1f80de697928820ec3f806cec67df136d6f8b6ff0ba7ef6f
#e4c228b37e7407becff9903198ebe2258d3774f198de8d0366cb10496408
#57dff62f247dfcae9813fde4167c1f959b291490cfb8e0b8db3395ce019f
#07c94aa4a86d1a0dae2b6cc6d139db740841dc65add58512ea59336dbc8c
#9abc14d500e42c8ea5031c201eec027d14e9878974629d2fdbe0d0cfffab
#8ab73f148bdec788de59550443ccf12cb3e74518274c3d7d2892eeb90f23
#0d5e853b85542c05b12198c097c2fb4023b370f0629645e11552f5429fb8
#aacef60a9344b5318202823082027e0201013081a6308198310b30090603
#55040613024341311230100603550408130963616c69666f6e6961311130
#0f060355040713086d69706c69746173311b3019060355040a1312636973
#636f2073797374656d7320696e632e310e300c060355040b13054e535354
#47311630140603550403130d436872697374696e652059616e311d301b06
#092a864886f70d010901160e6379616e40636973636f2e636f6d020900d4
#8b1bbfb069e9b9300906052b0e03021a0500a081b1301806092a864886f7
#0d010903310b06092a864886f70d010701301c06092a864886f70d010905
#310f170d3039303932393137343034335a302306092a864886f70d010904
#31160414374b75746dc97e2ccd5c6b5e430fb5c988a4ff7c305206092a86
#4886f70d01090f31453043300a06082a864886f70d0307300e06082a8648
#86f70d030202020080300d06082a864886f70d0302020140300706052b0e
#030207300d06082a864886f70d0302020128300d06092a864886f70d0101
#0105000482010066d87dc724b292156a3d2d887df39b7ae097ab822e36a3
#9286b8309b9cb21282de98fb7ac1ef13c936014d0412af3c1a60b1034161
#1f1a4ddb324f905db397e92d8b1035d3899f2cceea6483ed1b660e8a7385
#875f43e2f4a96264ce27b9db20d467c1986e027a37d99dbc74a13a11864b
#af9b1d9fdc81f2b2de084ea2d8f8b6a3948d1b90c01569cc22217ea42dc0
#e32093d04b7a6e18f272cf0f375c2f5fc8f0764d3fe35399101fceb7b822
#55ed2cd17f80342caf57931be1c72482df8ed251e30ab89e56686927fd3d
#90ee9a7ff1a23cd161b6c3f9644554c7d20243c24da68ecc88ab92d20ee8
#b60d3838bb5ca7869fe468ce468f976cb793cd60911ca9
