#------------------------------------------------------------------
#
# Mar. 2009 - Mike Cheng (mcheng1@cisco.com)
#             Junekang Yang (jkyang@cisco.com)
#
# Copyright (c) 2009 by cisco Systems, Inc.
# All rights reserved.
#
# Version-date: v1.0-03/10/2009
#
# Name: lib_2.tcl
#
# Desc: lib procedures to be shared by eem policies
#
#------------------------------------------------------------------

################################################################################
#
#    Description: debug messages
#
#    IN: trace: trace level - 0 no debug
#        msg: debug messages
#
#    OUT: none
#
################################################################################

proc debug {trace msg {level "1"}} {

    if {$trace >= $level} {
        puts $msg
    }
}

##############################################################################
#
#   Description: Config ios command at config mode.
#
#   IN: ios configuration cli
#   OUT: NONE
#
##############################################################################

proc ios_config_cmds { config_cmd_list log } {

    if [catch {cli_open} result] {
        error $result "Cannot open cli"
    } else {
        array set cli1 $result
    }


    if [catch {cli_exec $cli1(fd) "enable"} result] {
        error $result "Cannot go to enable mode"
	return 1
    }


    if [catch {cli_exec $cli1(fd) "conf t"} result] {
        error $result "Cannot go to configuration mode"
	return 1
    } else {
        upvar $log log2
        foreach config_cmd $config_cmd_list {
            if [catch {cli_exec $cli1(fd) $config_cmd} result] {
                error $result "Error in config command"
	        return 1
            }
            append log2 "\n\n" [clock format [clock seconds]] " " \
                        $config_cmd 
        }
    }

    if [catch {cli_exec $cli1(fd) "end"} result] {
        error $result "End of the cli"
        return 1
    }

    if [catch {cli_close $cli1(fd) $cli1(tty_id)} result] {
        error $result "Close cli"
        return 1
    }
    return 1
}

##############################################################################
#
#   Description: Config ios command at config mode. 
#  
#   IN: ios configuration cli
#   OUT: NONE
#
##############################################################################

proc ios_config { config_cmd {log ""}} {

    if [catch {cli_open} result] {
        error $result "Cannot open cli"
    } else {
        array set cli1 $result
    }


    if [catch {cli_exec $cli1(fd) "enable"} result] {
        error $result "Cannot go to enable mode"
    }


    if [catch {cli_exec $cli1(fd) "conf t"} result] {
        error $result "Cannot go to configuration mode"
    } else {
        if [catch {cli_exec $cli1(fd) $config_cmd} result] {
            error $result "Error in config command"
        } else {
            if {[string length $log] != 0} {
                upvar $log log2
                append log2 "\n\n" [clock format [clock seconds]] " "\
                             $config_cmd
            }
        }
    }

    if [catch {cli_exec $cli1(fd) "end"} result] {
        error $result "End of the cli"
    }

    if [catch {cli_close $cli1(fd) $cli1(tty_id)} result] {
        error $result "Close cli"
    }
}


###############################################################################
#
#   Description: IOS super user command.
#                This is for getting ios super command under enable mode.
#                such as show command etc.
#
#   IN: ios cli command which needs to be executed.
#   OUT: None.
#
###############################################################################

proc ios_enable_command2 { ios_command {log ""}} {

    if [catch {cli_open} result] {
        error $result "Cannot open file channel for enable command"
    } else {
        array set cli1 $result
    }

    # From the following cli, we can get fd for show command.
    if [catch {cli_exec $cli1(fd) "enable"} result] {
        error $result "Cannot go to enable mode"
    }

    if [catch {cli_exec $cli1(fd) "term len 0"} result] {
        error $result "Cannot configure terminal mode"
    }

    if [catch {cli_exec $cli1(fd) $ios_command } result] {
        error $result "Cannot exec ios command"
    } else {
        set cmd_out $result
        if {[string length $log] != 0} {
            upvar $log log2
            append log2 "\n\n" [clock format [clock seconds]] " " $ios_command \
        }
    }

    if [catch {cli_close $cli1(fd) $cli1(tty_id)} result] {
        error $result "Cannot close exec channel"
    }

    return $cmd_out
 
} 

##############################################################################
#
#   Description: shut_down_interface
#
#   IN: interface_name
#   OUT: NONE
#
##############################################################################

proc shut_down_interface { interface_name cmd } {

    set interface_cmd [format "interface %s" $interface_name]
    set config_list [list $interface_cmd "shut"]

    upvar $cmd cmd2
    ios_config_cmds $config_list cmd2
}

##############################################################################
#
#   Description: no_shut_interface
#
#   IN: interface_name
#   OUT: NONE
#
##############################################################################
proc no_shut_interface { interface_name cmd } {

    global trace
 
    set interface_cmd [format "interface %s" $interface_name]
    debug $trace "interface commands: $interface_cmd"
    set config_list [list $interface_cmd "no shut"]

    upvar $cmd cmd2
    ios_config_cmds $config_list cmd2

}

#############################################################################
#
#    Description: Validates null input given by the user
#
#    IN: An empty string "" or a string that has some content
#    OUT: returns 1 if the interface is up, otherwise returns 0
#
#############################################################################

proc is_interface_up {name cmd} {

    global trace
    upvar $cmd cmd2

    set show_command [format "show interface %s stats" $name]
    debug $trace $show_command 4

    set output [ios_enable_command2 $show_command cmd2]
    debug $trace $output 4
    foreach line [split $output "\n"] {
        #### only read the first line of the output
        debug $trace "Stat: $line" 4
        set line2 [string trim $line]
        if {[string length $line2] == 0} {
            continue
        }
        if {[regexp {disabled} $line] ==1 } {
            return 0
        }
        break
    }
    return 1
}


##############################################################################
#
#  Description:   get_command_history_msg
#                 Get configuration change from router
#
#  IN: command history log 
#  OUT: The historic command history header and message 
#
##############################################################################

proc get_command_history_msg {cmd} {

    set msg_header "\n\nTHE COMMAND HISTORY is: "
    set msg $msg_header
    if {[string length $cmd] == 0} {
        append msg \n\n "none"
    } else {
        append msg $cmd
    }

    append msg \n\n "----------------------------------------------------"
    append msg "----------"
    return $msg

}

##############################################################################
#
#  Description:        send_result_by_email2
#
#  IN: config_history  if config history is needed
#      event_history   if event history is needed
#      command_history if command history is needed
#      command logs    command log history
#      message         summary message (call by reference) 
#      long_message    detailed message
#      _email_from   environment variable for sending email
#      _email_to     environment variable for sending email
#      _email_cc     environment variable for sending email
#      _email_server environment variable for sending email
#
#  OUT: Return 1 if is sent successful.  Otherwise, return 0 
#
##############################################################################

proc send_result_by_email2 { config_history event_history command_history command_log message long_message _email_from _email_to _email_cc _email_server } {

    global msg_delimiter
    global trace

    upvar $message msg

    debug $trace "short message: $msg" 3
    debug $trace "long message: $long_message" 3

    if {$_email_from == ""} {
        puts "Email can not be sent because no valid _email_from\
            value\$msg_delimiter"
        return 0
    }

    if {$_email_to == ""} {
        puts "Email can not be sent because no valid _email_from\
            value\$msg_delimiter"
        return 0
    }

    if {$_email_server == ""} {
        puts "Email can not be sent because no valid _email_server\
            value\$msg_delimiter"
        return 0
    }

    append msg "\n\n"
    if {[string length $long_message] != 0} {
        append msg $long_message
        append msg "\n\n"
    }

    if { $config_history == "TRUE" } {
        set history_msg [get_histroy_msg]
        append msg $history_msg
    }

    if { $event_history == "TRUE" } {
        set event_history_msg [get_event_histroy_msg]
        append msg $event_history_msg
    }

    if { $command_history == "TRUE" } {
        set command_history_msg [get_command_history_msg $command_log]
        append msg $command_history_msg
    }

    debug $trace "Sending email to $_email_to from $_email_from" 2
    debug $trace "$msg" 2

    return [send_email_result $msg $_email_from $_email_to $_email_cc $_email_server]

}

proc prepend_syslog {trigger_msg sdiag_msg} {
     upvar $sdiag_msg prepended_msg
 
     set final_msg ""
 
     set new_trigger_msg [string trim $trigger_msg "\n"]
 
     append final_msg "TRIGGERED_BY \"$new_trigger_msg\":: " $prepended_msg
 
     set prepended_msg $final_msg
}

#############################################################################
#
#    Description: Send a notification string through EEM SNMP Trap
#
#    IN:  A notification string for SNMP Trap
#    OUT: It returns 0 if the EEM SNMP Trap action function accepted the
#         request. Otherwise, returns the non-zero value of _cerrno.
#
#############################################################################
 
proc send_snmp_trap { trap_str } {

    global _cerrno
    global trace
    debug $trace "Sending snmp trap..." 2
    debug $trace "$trap_str" 2

    # invoke the eem snmp trap service
    action_snmp_trap strdata $trap_str
    if {$_cerrno != 0} {
        # action_snmp_trap returned error
        set result [format "component=%s; subsys err=%s; posix err=%s;\n%s" \
            $_cerr_sub_num $_cerr_sub_err $_cerr_posix_err $_cerr_str]
        puts "ERROR: EEM action_snmp_trap failed. $result"
    }
    return $_cerrno
}

#############################################################################
#
#    Description: Get Email Parameters 
#
#    IN:  email_from: call by reference
#         email_to: call by reference
#         email_cc: call by reference
#         email_server: call by reference
# 
#    OUT: Return email related parameters from event manager 
#
#############################################################################

proc get_email_parameters { email_from email_to email_cc email_server } {
    global trace

    upvar $email_from email_from2
    upvar $email_to email_to2
    upvar $email_cc email_cc2
    upvar $email_server email_server2
    set cmd2 ""

    set result [ios_enable_command2 "show run | i event" cmd2]
    set code 1
    foreach line [split $result "\n"] {
        if {[regexp {event manager environment _email_from (.*)} $line match temp] \
            == 1} {
            set email_from2 [string trim $temp]  
            debug $trace "email_from: $email_from2"
        } elseif {[regexp {event manager environment _email_to (.*)} $line match temp] \
            == 1} {
            set email_to2 [string trim $temp]  
            debug $trace "email_to: $email_to2"
        } elseif {[regexp {event manager environment _email_cc (.*)} $line match temp] \
            == 1} {
            set email_cc2 [string trim $temp]  
            debug $trace "email_cc: $email_cc2"
        } elseif {[regexp {event manager environment _email_server (.*)} \
            $line match temp]  == 1} {
            set email_server2 [string trim $temp]  
            debug $trace "email_server: $email_server2"
        }
    }
    if {[string length $email_from2] == 0} {
        puts "ERROR: _email_from is not defined"
        set code 0
    }
    if {[string length $email_to2] == 0} {
        puts "ERROR: _email_to is not defined"
        set code 0  
    } 
    if {[string length $email_server2] == 0} { 
        puts "ERROR: _email_server is not defined"
        set code 0 
    } 
    if {$code == 0} {
        puts "WARNING: No email will be sent"
    }
    return $code 
}

################################################################################
#    Description: Send notification based on notification method
#
#    IN:  message: the summary message to be sent via notification
#         long_messsage: the detailed message if applicable
#                        it will be skipped for snmp and syslog 
#         notif:   Notification method - email, syslog, snmp, all
#         config - TRUE if it is enabled, FALSE if not
#         event  - TRUE if it is enabled, FALSE if not
#         command - TRUE if it is enabled, FALSE if not
#         cmd - command history
#         prepend - prepend triggered message [TRUE/FALSE]
#         trigger_msg - triggered message
#         automation - TRUE if is enabled, FALSE if not
#
#    OUT: Return 1 if all notifications are sent, Otherwise, return 0
#
################################################################################

proc send_notification {message long_message notif2 config event command cmd prepend trigger_msg automation} {
    global trace

    set email 0
    set snmp 0
    set syslog 0

    set notif [string tolower $notif2]
    debug $trace "enter send_notification method: $notif2"

    if {$notif == "email"} {
        debug $trace "email is 1"
        set email 1
    } elseif {$notif == "all"} {
        debug $trace "all is 1"
        set email 1
        set snmp 1
        set syslog 1
    } elseif {$notif == "snmp"} {
        debug $trace "snmp is 1"
        set snmp 1
    } elseif {$notif == "syslog"} {
        debug $trace "syslog is 1"
        set syslog 1
    } elseif {[string first "," $notif ] > 0} {
        debug $trace "notify has comma in it"
        foreach token [split $notif ","] {
            set token2 [string trim $token]
            if {$token2 == "email"} {
               debug $trace "email is 1"
               set email 1
            } elseif {$token2 == "syslog"} {
               debug $trace "syslog is 1"
               set syslog 1
            } elseif {$token2 == "snmp"} {
               debug $trace "snmp is 1"
               set snmp 1
            }
        }
    }

    if {($email == 0) && ($snmp == 0) && ($syslog == 0)} {
        puts "ERROR: Invalid notification method: $notif"
        return 0
    }
  
    set _email_from ""
    set _email_to ""
    set _email_cc ""
    set _email_server ""

    ### Prepend does not apply to snmp notification method because of
    ### 255 payload limitation
    set msg_length [string length $message]
    debug $trace "Before prepend message length: $msg_length" 3
    if {$snmp == 1} {
        send_snmp_trap $message
    }

    if {$prepend == "TRUE"} {
        prepend_syslog $trigger_msg message
    }

    set msg_length [string length $message]
    debug $trace "After prepend message length: $msg_length" 3
    if {$msg_length > 683} {
        if {$syslog == 1} {
            puts $message
            puts "WARNING: Above message length: $msg_length is longer than\
                  683 chars may be truncated using syslog notification\
                  method."
        }
    }

    if {$syslog == 1} {
        debug $trace "Sending syslog message.." 2
        debug $trace $message 2
        action_syslog msg $message
    }

    if {$email == 1} {
       if {[get_email_parameters _email_from _email_to _email_cc _email_server] != 1} {
           return 0
       }
       if {[send_result_by_email2 $config $event $command $cmd message $long_message \
           $_email_from $_email_to $_email_cc $_email_server] == 1} {
            if {$automation == "TRUE"} {
                puts "EMAIL: $message"
            }
       }
    }

    return 1
}

################################################################################
#
#    Description: Validate the interface name
#
#    IN: interface name
#
#    OUT: 1 is successful.  Otherwise, returns 0
#
################################################################################

proc is_valid_interface {interface_name cmd} {

    global trace

    upvar $cmd cmd2
    set ios_command "show interface $interface_name summary"

    set output [ios_enable_command2 $ios_command cmd2]
    foreach line [split $output "\n"] {
        debug $trace $line
        if {[string match "Invalid input" $line]} {
           puts "ERROR: Invalid interface name: $interface_name"
           return 0
        }
    }

    debug $trace "Interface name: $interface_name is valid"
    return 1
}

################################################################################
#
#    Description: validate policy generic environment variables 
#
#    IN: prefix: policy prefix
#        cmd: command history - call by reference 
#        automation: call by reference - return [TRUE/FALSE]
#
#    OUT: 0 is successful.  Otherwise, returns 1                                
#
################################################################################

proc check_policy_env {prefix cmd automation} {

    global trace
    upvar $cmd cmd2
    upvar $automation auto

    set result [ios_enable_command2 "show event manager environ" cmd2]
    #debug $trace "Result:"
    #debug $trace "$result"

    set notif_env $prefix
    append notif_env "Notif"
    set config_env $prefix
    append config_env "Config"
    set event_env $prefix
    append event_env "Event"
    set command_env $prefix
    append command_env "Command"
    set prepend_env $prefix
    append prepend_env "Prepend"

    set notif ""
    set config ""
    set event ""
    set command ""
    set prepend ""

    set error 0
    set auto FALSE

    foreach line [split $result "\n"] {
        if {[regexp {_svcdiag_(\S*)(\s*)(.*)} $line match temp temp2 \
             temp3] == 1} {
           if {[string first $notif_env $line] >= 0} {
               set notif [string trim $temp3]
           } elseif {[string first $config_env $line] >= 0 } {
               set config [string trim $temp3]
           } elseif {[string first $event_env $line] >= 0 } {
               set event [string trim $temp3]
           } elseif {[string first $command_env $line] >= 0 } {
               set command [string trim $temp3]
           } elseif {[string first $prepend_env $line] >= 0 } {
               set prepend [string trim $temp3]
           }
        } elseif {[regexp {_automation(\s*)(.*)} $line match temp temp1] == 1} {
           debug $trace "_automation: $temp1"
           set value [string toupper [string trim $temp1]]
           if {$value == "TRUE"} {
               set auto TRUE
           }
        } 
    }

    debug $trace "notif: $notif"
    debug $trace "config: $config"
    debug $trace "event: $event"
    debug $trace "command: $command"
    debug $trace "prepend: $prepend"
    debug $trace "automation: $auto"

    set policy 1
    set error [validateParameters $notif $config $event $command \
                                  $prepend $policy]
    if {$error != 0} {
        puts "ERROR: Validation errors on generic policy environment settings"
    }
    return $error
}

################################################################################
#
#    Description: exit policy script with error messages 
#
#    IN: script: policy script name 
#        message: error message [optional] 
#
#    OUT: None 
#
################################################################################

proc exit_policy_script_with_errors { script {message ""}} {

    if {[string length $message] != 0} {
        puts "ERROR: $message"
    }
    puts "\nERROR: Aborting from policy script - $script..."
    puts "\n\n"

}
