
#------------------------------------------------------------------------------
# User library to store all the procedures used by different diagnostic scripts 
#
# June 2007 - Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007-2009 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-06/20/2007 
# Title: Store(configure) inputs from user and register with EEM.
# 
# Name: userlib.tcl
#
# Desc: Stores all the procedures most of which does the validation of the
#       inputs given by the user  
#
#-------------------------------------------------------------------

#############################################################################
#
#    Description: Validates null input given by the user
#
#    IN: An empty string "" or a string that has some content  
#    OUT: returns 1 if the input is "" otherwise returns 0 
#  
#############################################################################

proc checkNullInput {value} {
   if {$value == ""} {
       return 1 
   } else {
       return 0
   }
}

#############################################################################
#
#    Description: Validates the high and low threshold values input by the user
#
#    IN: A string which contains the high or the low threshold value    
#    OUT: returns 0 if the threshold value is within the range 0 - 100
#         otherwise returns 1 
#  
#############################################################################

proc checkRange {value} {
    if {([regexp {[^0-9]+} $value] == 0)} {
        regexp {[0-9]+} $value a
        if {$a > 0 && $a < 100} {
            return 0
        } else {
            return 1
        }
    } else {
        return 1
    }
}

#############################################################################
#
#    Description: Validates the time interval input by the user for the 
#                 resource moniotring scenarios
#
#    IN: The time interval input by the user     
#    OUT: returns 1 if the time interval is not within the range 5 - 86400
#         otherwise returns 0 
#  
#############################################################################

proc checkIntervalLimit {value} {
    if {($value < 5) || ($value > 86400)} {    
        set msg "\n\nSERVICE-DIAGNOSTICS ERROR: Invalid time interval "
        append msg $value
        append msg "\n\nPlease enter a value between 5 and 86400"
        puts stderr $msg
        return 1
    } else {
        return 0
    }
}
 
#############################################################################
#
#    Description: Validates the input for notification given by the user 
#
#    IN: A string which contains "email" or "syslog" or some other value     
#    OUT: returns 1 if the input string is not "email" or "syslog" 
#         otherwise returns 0 
#  
#############################################################################

proc checkNotification {value} {
    if {($value != "email") && ($value != "syslog") && ($value != "all")} {
        return 1
    } else {
        return 0
    }
}

#############################################################################
#
#    Description: Validates the inputs given for additional diagnoses  
#
#    IN: A string which contains the values "TRUE" or "FALSE" or some other
#        value     
#    OUT: returns 1 if the input string is not "TRUE" or "FALSE" 
#         otherwise returns 0 
#  
#############################################################################

proc checkFlag {value} {
    if {($value != "TRUE") && ($value != "FALSE")} {
        return 1
    } else {
        return 0
    } 
}

#############################################################################
#
#    Description:  Validates the inputs specified for the additonal diagnoses
#                  to be done   
#
#    IN: "TRUE" or "FALSE" flag values  
#    OUT: returns error message if valid inputs are not provided otherwise
#         returns from the procedure after complete execution of the procedure 
#  
#############################################################################

proc validateOtherInputs {notif config event} {

    set error 0
    if {[checkNotification $notif] == 1} {
        puts "The value you entered for 'notification' is $notif"
        puts "Enter 'email' or 'syslog' or 'all' as input\n"
        set error 1
     } 
   
    if {([checkFlag $config] == 1) || ([checkFlag $event] == 1)} {
        puts "The value you entered for configurationHistory is $config"
        puts "The value you entered for eventHistory is $event"
        puts "Enter either 'TRUE' or 'FALSE' as input" 
        set error 1
    }
    return $error
}

#############################################################################
#
#    Description: Validates the time interval input by the user for the 
#                 eem policy scripts that are triggered after the timer 
#                 expiration 
#
#    IN: The time interval input by the user     
#    OUT: returns error message if the time interval is not within the range 
#         0 - 4294967295 otherwise returns after executing the procedure
#  
#############################################################################

proc validateEEMPolicyTimer {time} {

    if {[string first . $time] == -1} {
        append time ".0"
    }

    if {[string is double $time] != 1} {
        puts "Please enter a valid integer in the range (1 - 4294967295)"
        return 1
    } elseif {($time < 1.0) || ($time > 4294967295.0)} {
        puts "The value you entered is $time"
        puts "Please enter a valid value in the range (1 - 4294967295)"
        return 1
    } else {
        set var [split $time .]
        set decimal [lindex $var 1]
        if {![string equal $decimal "0"] || ([string length $decimal] > 1)} {
            puts "Floating point values are not allowed, truncating to integer"
        }
        return [lindex $var 0]
    }
}
 
#############################################################################
#
#    Description: Finds out if inputs related to email was already given by
#                 the user 
#
#    IN: The string that has the value "email" or "syslog"     
#    OUT: returns empty string "" if the email inputs was not provided by thei
#         user earlier otherwise returns configured 
#  
#############################################################################

proc emailInputAlreadyGiven {notification} {
    
    set msgstr ""
    set from ""
    set to ""
    set cc ""
    set server ""
    set result [exec "show run | i event"]

    foreach line [split $result "\n"] {
        if {[regexp {event manager environment _email_from (.*)} $line] == 1} {
            set from 1
        }
        if {[regexp {event manager environment _email_to (.*)} $line] == 1} {
            set to 1
        } 
        if {[regexp {event manager environment _email_cc} $line] == 1} {
            set cc 1
        }
        if {[regexp {event manager environment _email_server (.*)} $line]\
            == 1} {
            set server 1
        }
    }

    if {$from != 1} {
        puts "ERROR: No input for \"_email_from\" is given"
    } 
    if {$to != 1} {
        puts "ERROR: No input for \"_email_to\" is given"
    } 
    if {$cc != 1} {
        puts "ERROR: No input for \"_email_cc\" is given"
    } 
    if {$server != 1} {
        puts "ERROR: No input for \"_email_server\" is given"
    } 

    if {($from == 1) && ($to == 1) && ($cc == 1) && ($server == 1)} {
        set msgstr "configured"
    }
    return $msgstr
}

#############################################################################
#
#    Description: Displays all the inputs configured on the router based on the
#                 lists passed to this procedure 
#
#    IN: List that contains inputs related to a particular scenario   
#    OUT: Displays all the configured inputs in a format easily readable by the 
#         user
#  
#############################################################################

proc displayEnvVariables {List type} {
    puts "\nTHE INPUTS GIVEN FOR $type ARE:"
    foreach index $List {
        switch -regexp -- $index {
            Alloc {
                set alloc [split $index :]
                puts "\tResource allocation option : [lindex $alloc 1]"
            }
            Auto {
                set auto [split $index :]
                puts "\tAuto configuration option : [lindex $auto 1]"
            }
            cc {
                set cc [split $index :]
                puts "\tCC ID : [lindex $cc 1]"
            }
            Config {
                set config [split $index :]
                puts "\tConfiguration history option : [lindex $config 1]"
            }
            Event {
                set event [split $index :]
                puts "\tEvent history option : [lindex $event 1]"
            }
            from {
                set from [split $index :]
                puts "\tFrom ID : [lindex $from 1]"
            }
            FormTimer {
                set neigh [split $index :]
                puts "\tTimer value for Neighbor Formation Problems is set to : [lindex $neigh 1]"
            }
            Notif {
                set notif [split $index :]
                puts "\tNotification : [lindex $notif 1]"
            }
            RouteTimer {
                set route [split $index :]
                puts "\tTimer value for Route Problems is set to : [lindex $route 1]"
            }
            server {
                set server [split $index :]
                puts "\tEmail Server : [lindex $server 1]"
            }
            to {
                set to [split $index :]
                puts "\tTo ID : [lindex $to 1]"
            }
            default {
                puts "\tNO CONFIGURATION"
            }
        }
    }
}

#############################################################################
#
#    Description: Executes "show event manager environment command and parses
#                 the output for configuration corresponding to the inputs
#                 entered by the user and displays them in a format easy for the#                 user to understand  
#
#    IN: NONE   
#    OUT: Stores the parsed result in lists and calls the procedure 
#         "displayEnvVariables" 
#  
#############################################################################

proc showEEMVariables {type} {

### Initialize list which will store the configured inputs ###
    
    set bgpLoss {}
    set bgpForm {}
    set bgpRoute {}

    set bgpLossOption "FALSE"
    set bgpFormOption "FALSE"
    set bgpRouteOption "FALSE"

### check validity of non-interactive inputs and give error message ###

    set msg "\n\nSERVICE-DIAGNOSTICS ERROR: Input not valid"
    append msg "\nThe value you entered is: "

    if {($type != "bgp_neighbor_loss.tcl") && ($type != "bgp_neighbor_formation.tcl") && ($type != "bgp_neighbor_route.tcl")} {
        puts "$msg $type"
        puts "\nEnter either <bgp_neighbor_loss.tcl> or <bgp_neighbor_formation.tcl> or <bgp_neighbor_route.tcl>"
        return
    }

    
    switch -regexp -- $type {
        
        bgp_neighbor_loss.tcl {
            set bgpLossOption "TRUE"
        }
        bgp_neighbor_formation.tcl {
            set bgpFormOption "TRUE"
        }
        bgp_neighbor_route.tcl {
            set bgpRouteOption "TRUE"
        }
        default {
            puts "No match"
        }
    }

    if {[catch {exec "show event manager environment"} result] != 0} {
        puts stderr "'show event manager environment' did not execute"
    } else {
        set output [split $result "\n"]
    
        foreach line $output {
            set envVar [lindex $line 1]
            set value [lindex $line 2]
        
            if {[regexp {_email} $envVar] == 1} {
                lappend emailParam $envVar:$value
            } else {
                switch -regexp -- $envVar {
                    bgpLoss {
                        lappend bgpLoss $envVar:$value
                    }
                    bgpForm {
                        lappend bgpForm $envVar:$value
                    }
                    bgpRoute {
                        lappend bgpRoute $envVar:$value
                    }
                    default {
                        puts ""
                    }
                }
            }
        }

        if {$bgpLossOption == "TRUE"} {
            if {[llength $bgpLoss] != 0} {
                displayEnvVariables $bgpLoss "BGP NEIGHBOR LOSS PROBLEM"
            } else {
                puts "NO INPUTS WERE GIVEN FOR THE BGP NEIGHBOR LOSS PROBLEM\n"
                return 1
            }
        }

        if {$bgpFormOption == "TRUE"} {
            if {[llength $bgpForm] != 0} {
                displayEnvVariables $bgpForm "BGP NEIGHBOR FORMATION PROBLEM"
            } else {
                puts "NO INPUTS WERE GIVEN FOR THE BGP NEIGHBOR FORMATION PROBLEM\n"
                return 1
            }
        }

        if {$bgpRouteOption == "TRUE"} {
            if {[llength $bgpRoute] != 0} {
                displayEnvVariables $bgpRoute "BGP ROUTE PROBLEM"
            } else {
                puts "NO INPUTS WERE GIVEN FOR THE BGP ROUTE PROBLEM\n"
                return 1
            }
        }
    }
}



#Cisco Tcl Signature V1.0
#3082077506092a864886f70d010702a082076630820762020101310b3009
#06052b0e03021a0500300b06092a864886f70d010701a08204bb308204b7
#3082039fa003020102020900d48b1bbfb069e9b9300d06092a864886f70d
#0101050500308198310b3009060355040613024341311230100603550408
#130963616c69666f6e69613111300f060355040713086d69706c69746173
#311b3019060355040a1312636973636f2073797374656d7320696e632e31
#0e300c060355040b13054e53535447311630140603550403130d43687269
#7374696e652059616e311d301b06092a864886f70d010901160e6379616e
#40636973636f2e636f6d301e170d3037313231383138333232375a170d31
#30313231373138333232375a308198310b30090603550406130243413112
#30100603550408130963616c69666f6e69613111300f060355040713086d
#69706c69746173311b3019060355040a1312636973636f2073797374656d
#7320696e632e310e300c060355040b13054e535354473116301406035504
#03130d436872697374696e652059616e311d301b06092a864886f70d0109
#01160e6379616e40636973636f2e636f6d30820122300d06092a864886f7
#0d01010105000382010f003082010a02820101009c22f1d01af397756ab6
#cfce92a9e761b73fbf1efd070aa4e90448e6077e2c127ff47a5951de7790
#55da014ff152576a08d7199b8ad5757eb8c9a7c1a493715df2176e9f56f0
#74735d43420cf1ed17fe0aa3494724808aefcf274572c7750be071a94560
#9c045c48a5454f9e7933bf93d967bcf582918b1bbd48f47dd978c7ea9c88
#23eb01482a40ef770f823ea2e63eaf7cab5367bd51b54979925464e27b22
#82fc5bfe94f6f41f2f91e56444ae371a050107fe79fd5db3f228bbbd5fcb
#653a87a3c1268d74500ba1489382290804d9d7937d3269b1c55eae51f9f7
#62eadbd4226ba6d4008414b353181ee6cfbe24204e4a82436b117c255014
#77ec74c3ac4d0203010001a38201003081fd301d0603551d0e041604149c
#0e12d3a222891fc1300b780f38b54e07e228073081cd0603551d230481c5
#3081c280149c0e12d3a222891fc1300b780f38b54e07e22807a1819ea481
#9b308198310b300906035504061302434131123010060355040813096361
#6c69666f6e69613111300f060355040713086d69706c69746173311b3019
#060355040a1312636973636f2073797374656d7320696e632e310e300c06
#0355040b13054e53535447311630140603550403130d436872697374696e
#652059616e311d301b06092a864886f70d010901160e6379616e40636973
#636f2e636f6d820900d48b1bbfb069e9b9300c0603551d13040530030101
#ff300d06092a864886f70d010105050003820101003e6d657e30c21acb30
#d7e91aef99cd6f83a8644d7ae437fb4a0cd1a8c6a0f1cb2194be80868b75
#1d20ee8dc72a1f80de697928820ec3f806cec67df136d6f8b6ff0ba7ef6f
#e4c228b37e7407becff9903198ebe2258d3774f198de8d0366cb10496408
#57dff62f247dfcae9813fde4167c1f959b291490cfb8e0b8db3395ce019f
#07c94aa4a86d1a0dae2b6cc6d139db740841dc65add58512ea59336dbc8c
#9abc14d500e42c8ea5031c201eec027d14e9878974629d2fdbe0d0cfffab
#8ab73f148bdec788de59550443ccf12cb3e74518274c3d7d2892eeb90f23
#0d5e853b85542c05b12198c097c2fb4023b370f0629645e11552f5429fb8
#aacef60a9344b5318202823082027e0201013081a6308198310b30090603
#55040613024341311230100603550408130963616c69666f6e6961311130
#0f060355040713086d69706c69746173311b3019060355040a1312636973
#636f2073797374656d7320696e632e310e300c060355040b13054e535354
#47311630140603550403130d436872697374696e652059616e311d301b06
#092a864886f70d010901160e6379616e40636973636f2e636f6d020900d4
#8b1bbfb069e9b9300906052b0e03021a0500a081b1301806092a864886f7
#0d010903310b06092a864886f70d010701301c06092a864886f70d010905
#310f170d3039303932383233303834305a302306092a864886f70d010904
#3116041467bad09ab14d105a4aba0e7be401381f050242c9305206092a86
#4886f70d01090f31453043300a06082a864886f70d0307300e06082a8648
#86f70d030202020080300d06082a864886f70d0302020140300706052b0e
#030207300d06082a864886f70d0302020128300d06092a864886f70d0101
#010500048201000c05b88d45b7bab9583b30d0908cbf35d953946d28bbd4
#45959ac16656527eb8142b49e66c940ee1d0e79a80627c19e0c0d7071cf9
#ac8825a6e552cd53070222a4220b12c3c4f643245e07803248f2afbb429b
#eb7adb29ad450c480f8f1e94972e141ea5f0da672b493e999a32bd60d857
#0e472f9cb7e493fcafb14a7dc22fd23cb0a40111978ead183f324e3084c5
#d539a5303689295217ce3c7571415721034b2197a1502349414501d9d853
#b8271f2525da91e4337ef0dbe4c26b954fa1425c0f7131db45bf55baece1
#ef7c813e87e1a445247e15b62acc34befbc345b0cbcf689130003b950d8f
#6b0df9989be538de65e52fd3eaf85f7ef1903124dadfb3
