
#------------------------------------------------------------------------------
# User library to store all the procedures used by different diagnostic scripts 
#
# Mar 2009 - Mike Cheng (mcheng1@cisco.com)
#            Junekang Yang (jkyang@cisco.com)
#            Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007-2009 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-03/10/2007 
# Title: Store(configure) inputs from user and register with EEM.
# 
# Name: userlib_2.tcl
#
# Desc: Stores all the procedures most of which does the validation of the
#       inputs given by the user  
#
#-------------------------------------------------------------------

#############################################################################
#
#    Description: Validates null input given by the user
#
#    IN: An empty string "" or a string that has some content
#    OUT: returns 1 if the input is "" otherwise returns 0
#
#############################################################################

proc logging {trace msg {level "1"}} {

    if {$trace >= $level} {
        puts $msg
    }
}

#############################################################################
#
#    Description: Check SNMP Trap setting on the device
#
#    IN:  None
#    OUT: It returns 0 if SNMP Trap is enabled for EEM and there is at least
#         one SNMP trap destination configured on the device.
#         Otherwise, returns non-zero value.
#
#############################################################################
 
proc check_snmp_trap_setting {policy} {

    # check the two essential conditions
    if {$policy == 0} {
        set snmp_trap_eem [string first snmp-server \
        [exec {sh run | i snmp-server enable traps event-manager}]]
        set snmp_trap_host [string first snmp-server \
        [exec {sh run | i snmp-server host}]]
    } else {
        set snmp_trap_eem [string first snmp-server \ 
        [ios_enable_command2 {sh run | i snmp-server enable traps event-manager}]]
        set snmp_trap_host [string first snmp-server \ 
        [ios_enable_command2 {sh run | i snmp-server host}]]
    } 
    if {$snmp_trap_eem != 0} {
        # report eem snmp trap is not enabled at all
        puts {ERROR: Please enable EEM SNMP Trap by configuring\
           "snmp-server enable traps event-manager"}
    }
 
    if {$snmp_trap_host != 0} {
        # report there is no snmp trap hosts configured to receive snmp trap
        puts {ERROR: Please add SNMP Trap destination by configuring\
            "snmp-server host"}
    }
 
    return [expr $snmp_trap_eem + $snmp_trap_host] 
}

#############################################################################
#    Description: validate the notification methods and corresponding
#                 configuration
#
#    IN:   notif - notification method
#    OUT:  return 0 if validation is successful.  Otherwise return 1
#
##############################################################################

proc validateNotification {notif2 config event command policy} {

    global trace

    set err 0
    set email 0
    set snmp 0
    set syslog 0
    set err "\n\nERROR: Invalid notification method:"

    set notif [string tolower $notif2]
    if {($notif == "email")} {
        logging $trace "email is 1"
        set email 1
    } elseif {($notif == "all")} {
        logging $trace "notification is all"
        set email 1
        set snmp 1
        set syslog 1
    } elseif { ($notif == "snmp") } {
        logging $trace "snmp is 1"
        set snmp 1
    } elseif {($notif == "syslog")} {
        logging $trace "syslog is 1"
        set syslog 1
    } elseif {[string first "," $notif] > 0} {
        foreach token [split $notif ","] {
            set token2 [string trim $token]
            if {($token2 == "email")} {
               logging $trace "email is 1"
               set email 1
            } elseif {($token2 == "syslog")} {
               logging $trace "syslog is 1"
               set syslog 1
            } elseif {($token2 == "snmp")} {
               logging $trace "snmp is 1"
               set snmp 1
            } else {
               puts "$err $token2"
               return 1
            }
        }
    } else {
        puts "$err $notif"
        return 1
    }

    if {($email == 0) && ($snmp == 0) && ($syslog == 0)} {
        puts "$err $notif"
        return 1
    }

    if {($syslog == 1) || ($snmp == 1)} {
        if {($config == "TRUE") || ($event == "TRUE") || ($command == "TRUE")} {
            puts "\nWARNING: Configuration, Event, Command history cannot\
                   be sent along with syslog/snmp notification\n"
        }
    }

    if {($email == 1) && ($policy == 0)} {
        if {[emailInputAlreadyGiven  $notif] != "configured"} {
            puts "Run the file 'collectEmailParameters.tcl'"
            puts "Usage: tclsh disk#:/dir_name/collectEmailParameters.tcl email_fromID email_toID email_ ccID email_server name "
            return 1
        }
    }

    if {($snmp == 1) && ($policy == 0)} {
        if {[check_snmp_trap_setting $policy] != 0} {
            return 1 
        }
    }
    return 0
}

#############################################################################
#    Description:  Validates the inputs specified for the additonal diagnoses
#                  to be done
#
#    IN: "TRUE" or "FALSE" flag values
#    OUT: returns error message if valid inputs are not provided otherwise
#         returns from the procedure after complete execution of the procedure
#
#############################################################################

proc validateParameters {notif config2 event2 command2  prepend2 policy} {                             

    global trace
    set error 0
    set config [string toupper $config2]
    set event [string toupper $event2]
    set command [string toupper $command2]
    set prepend [string toupper $prepend2]

    if {[validateNotification $notif $config $event $command $policy] == 1} {
        set error 1
    }

    if {([checkFlag $config] == 1) || ([checkFlag $event] == 1) || 
        ([checkFlag $command] == 1) || ([checkFlag $prepend] == 1)} {
        puts "The value you entered for configurationHistory is: \"$config\""
        puts "The value you entered for eventHistory is: \"$event\""
        puts "The value you entered for commandHistory is: \"$command\""
        puts "The value you entered for prepend is: \"$prepend\""
        puts "Enter either 'TRUE' or 'FALSE' as input"
        set error 1
    }

    return $error
}



#Cisco Tcl Signature V1.0
#3082077506092a864886f70d010702a082076630820762020101310b3009
#06052b0e03021a0500300b06092a864886f70d010701a08204bb308204b7
#3082039fa003020102020900d48b1bbfb069e9b9300d06092a864886f70d
#0101050500308198310b3009060355040613024341311230100603550408
#130963616c69666f6e69613111300f060355040713086d69706c69746173
#311b3019060355040a1312636973636f2073797374656d7320696e632e31
#0e300c060355040b13054e53535447311630140603550403130d43687269
#7374696e652059616e311d301b06092a864886f70d010901160e6379616e
#40636973636f2e636f6d301e170d3037313231383138333232375a170d31
#30313231373138333232375a308198310b30090603550406130243413112
#30100603550408130963616c69666f6e69613111300f060355040713086d
#69706c69746173311b3019060355040a1312636973636f2073797374656d
#7320696e632e310e300c060355040b13054e535354473116301406035504
#03130d436872697374696e652059616e311d301b06092a864886f70d0109
#01160e6379616e40636973636f2e636f6d30820122300d06092a864886f7
#0d01010105000382010f003082010a02820101009c22f1d01af397756ab6
#cfce92a9e761b73fbf1efd070aa4e90448e6077e2c127ff47a5951de7790
#55da014ff152576a08d7199b8ad5757eb8c9a7c1a493715df2176e9f56f0
#74735d43420cf1ed17fe0aa3494724808aefcf274572c7750be071a94560
#9c045c48a5454f9e7933bf93d967bcf582918b1bbd48f47dd978c7ea9c88
#23eb01482a40ef770f823ea2e63eaf7cab5367bd51b54979925464e27b22
#82fc5bfe94f6f41f2f91e56444ae371a050107fe79fd5db3f228bbbd5fcb
#653a87a3c1268d74500ba1489382290804d9d7937d3269b1c55eae51f9f7
#62eadbd4226ba6d4008414b353181ee6cfbe24204e4a82436b117c255014
#77ec74c3ac4d0203010001a38201003081fd301d0603551d0e041604149c
#0e12d3a222891fc1300b780f38b54e07e228073081cd0603551d230481c5
#3081c280149c0e12d3a222891fc1300b780f38b54e07e22807a1819ea481
#9b308198310b300906035504061302434131123010060355040813096361
#6c69666f6e69613111300f060355040713086d69706c69746173311b3019
#060355040a1312636973636f2073797374656d7320696e632e310e300c06
#0355040b13054e53535447311630140603550403130d436872697374696e
#652059616e311d301b06092a864886f70d010901160e6379616e40636973
#636f2e636f6d820900d48b1bbfb069e9b9300c0603551d13040530030101
#ff300d06092a864886f70d010105050003820101003e6d657e30c21acb30
#d7e91aef99cd6f83a8644d7ae437fb4a0cd1a8c6a0f1cb2194be80868b75
#1d20ee8dc72a1f80de697928820ec3f806cec67df136d6f8b6ff0ba7ef6f
#e4c228b37e7407becff9903198ebe2258d3774f198de8d0366cb10496408
#57dff62f247dfcae9813fde4167c1f959b291490cfb8e0b8db3395ce019f
#07c94aa4a86d1a0dae2b6cc6d139db740841dc65add58512ea59336dbc8c
#9abc14d500e42c8ea5031c201eec027d14e9878974629d2fdbe0d0cfffab
#8ab73f148bdec788de59550443ccf12cb3e74518274c3d7d2892eeb90f23
#0d5e853b85542c05b12198c097c2fb4023b370f0629645e11552f5429fb8
#aacef60a9344b5318202823082027e0201013081a6308198310b30090603
#55040613024341311230100603550408130963616c69666f6e6961311130
#0f060355040713086d69706c69746173311b3019060355040a1312636973
#636f2073797374656d7320696e632e310e300c060355040b13054e535354
#47311630140603550403130d436872697374696e652059616e311d301b06
#092a864886f70d010901160e6379616e40636973636f2e636f6d020900d4
#8b1bbfb069e9b9300906052b0e03021a0500a081b1301806092a864886f7
#0d010903310b06092a864886f70d010701301c06092a864886f70d010905
#310f170d3039303932353231303633385a302306092a864886f70d010904
#3116041458e2a6af6c45c006d066f0479f53c01a948d8c56305206092a86
#4886f70d01090f31453043300a06082a864886f70d0307300e06082a8648
#86f70d030202020080300d06082a864886f70d0302020140300706052b0e
#030207300d06082a864886f70d0302020128300d06092a864886f70d0101
#010500048201006794b5213cd4ac55cadfc1ed0686ed07348becfc946e4c
#ea9fd615b45f682a2a8227ca0d8c9ec314cf7be52fecc5ea3c7be4cfa5f3
#9323ca1894d655dea08a59ca0a952b9d96793c55f8f5dfb4cd5b04e3cc5c
#b594056fd80f78f91f6e9a844e4b93e8688e811a166ab5e59a6a97c127e0
#5b74cd03b876294b4f06636557c18174b43c52a1b31b34a809cb028792e5
#9403f1c04d3904551f6cba382013d9e400dfbc9d41e26063591c9bbc244f
#2211b057c54c0ecf98e0495cd3bb7aa8d272989457145d23e7ebf8e8d67b
#5cda48353d02a201bc7ab28b22ea9348232a7bfa2b3128b3620070d18034
#0ae19f87a43441d4e8598e62a7dc407632cb1447992373
