::cisco::eem::event_register_timer watchdog name auto_trace sec $_svcdiag_cfmAutotraceTimer  maxrun_sec 600 

#------------------------------------------------------------------------------
# EEM policy to perform Auto-trace 
#
# Dec 2008 - Mike Cheng (mcheng1@cisco.com)
#
# Copyright (c) 2008 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-12/01/2008 
# Title: Perform auto-trace for all endpoints for all specified MA names 
# Name: cfm_autotrace.tcl
# Desc: This script will perform the auto-trace for all endpoints based on specified MA names 
# 
# 
# Input-file:
#
# Environment Variables for this script are:
# _svcdiag_cfmAutotraceNotif: notification 
#                             [email|syslog|snmp|all] or any combinations  
# _svcdiag_cfmAutotraceConfig: config history [TRUE/FALSE]   
# _svcdiag_cfmAutotraceEvent: event history [TRUE/FALSE]   
# _svcdiag_cfmAutotraceCommand: command history [TRUE/FALSE] 
# _svcdiag_cfmAutotracePrepend: prepend triggered message [TRUE/FALSE] 
# _svcdiag_cfmAutotraceMAFile: input file which contains the MA names
# _svcdiag_cfmAutotraceTimer:  Frequence to run in secs.  sdiag_cfm.tcl will
#                              convert it from minutes to secs
# _automation: Test automation [TRUE/FALSE]  
# 
#------------------------------------------------------------------------------

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

### Initialize Command History and global variables
set cmd ""
set policy 1
global trace
set script "cfm_autotrace.tcl"

if {![info exists _debug_cfm_autotrace]} { 
    set _debug_cfm_autotrace 0   
}    
set trace $_debug_cfm_autotrace 

### Check generic policy environment variables
if {[check_policy_env "_svcdiag_cfmAutotrace" cmd automation] != 0} {
    exit_policy_script_with_errors $script 
    exit
}  

### Policy execution starts from here ###
debug $trace  "CFM Autotrace timer script is invoked..."

### Make sure the traceroute cache is enabled
set result ""
if {[is_traceroute_cache_enabled $policy cmd result] == 0} {
    set err "Trace route cache is not enabled"
    exit_policy_script_with_errors $script $err
    exit
}

### Display Cache size and hold-timer  ###
set hold_timer 0 
set size 0 
if {[display_traceroute_configuration $policy cmd hold_timer size]\
     == 0} {
    set err "Failed to parse traceroute configuration"
    exit_policy_script_with_errors $script $err
    exit
}

### Validate timer against hold_timer
### First convert the timer back to minutes
set timer [expr $_svcdiag_cfmAutotraceTimer / 60]
if {$hold_timer < $timer} {
    puts "CFM auto-trace timer: $timer mins"
    puts "\nWARNING: Adjust timer values such that hold-timer >= n * auto-trace\
          timer where n >= 1, n = 1.5 recommended\n" 
}

debug $trace "MA File: $_svcdiag_cfmAutotraceMAFile"

### Retrieve ma names from the file
set ma_list {}                                                                 
if {[get_ma_list $_svcdiag_cfmAutotraceMAFile ma_list] == 0} {  
    exit_policy_script_with_errors $script
    exit
}

debug $trace "MA List: $ma_list"

set policy 1

### Retrieve valid and invalid MA names by comparing the input MA names
### against the MA names from DB
set invalid_ma_list {} 
set valid_ma_list [get_valid_ma_list $ma_list invalid_ma_list cmd]
if {[llength $valid_ma_list] == 0} {
    set err "No valid MA names specified in the input file"
    exit_policy_script_with_errors $script $err
    exit
}

debug $trace "Valid MA names: $valid_ma_list"
debug $trace "Invalid MA names: $invalid_ma_list"

### Get all domains in the router ###
set domain_list [get_domain_list cmd $policy]
debug $trace "Domain List: $domain_list"

### Get Remote MEPID info ###
set summary ""
set details ""
traceroute_all_mpid $domain_list $valid_ma_list $_svcdiag_cfmAutotraceMAFile\
                    summary details cmd

### Print out the summary report
debug $trace "$summary\n\n" 3

### Print out the detailed report
debug $trace "$details\n\n" 3 

debug $trace "Cmd history:\n" 3
debug $trace $cmd 3

set timer_message "auto-trace watchdog timer every $timer minutes."
### Send out the summary and detailed message via notification
send_notification $summary $details $_svcdiag_cfmAutotraceNotif $_svcdiag_cfmAutotraceConfig $_svcdiag_cfmAutotraceEvent $_svcdiag_cfmAutotraceCommand $cmd $_svcdiag_cfmAutotracePrepend $timer_message $automation         

debug $trace "CFM autotrace script is done"
