::cisco::eem::event_register_syslog pattern "\%E_CFM-3-REMOTE_MEP_DOWN: Remote MEP mpid.*vlan.*MA.*name.*in.*domain.*changed.*state.*to.*down.*with.*event.*code.*TimeOut.*" maxrun 300

#------------------------------------------------------------------------------
# EEM policy to diagnose and attempt to isolate the location of network
# failures that break connectivity of Layer-2 Ethernet service as detected by
# the CFM Continuity Check (CC) protocol. The trigger is a syslog message.
#
# Feb 2009 - Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2009 by cisco Systems, Inc.
# All rights reserved.
#
# Version-date: v1.0 02/26/2009
# Title: Reports all the information related to the remote mep that has timed
#        out.
# Name: cfm_cctimeout.tcl
# Desc: This script will collect information related to the remote mep that 
#       timed out as indicated by the syslog message and perform further
#       diagnoses to collect more information to send it to the user. 
#
#       The structure and requirment of the file is defined below.
#
#       For the purpose of test automation an environment variable _automation
#       is required. This environment variable is checked if defined
#       previously and if not is defined.
#
# Environment Variables for this script are:
# _svcdiag_cfmCctimeoutNotif: notification - email/syslog/snmp/all
# _svcdiag_cfmCctimeoutConfig: configuration history - true/false
# _svcdiag_cfmCctimeoutEvent: event history - true/false
# _svcdiag_cfmCctimeoutCommand: command history - true/false
# _svcdiag_cfmCctimeoutPrepend: prepend trigger message - true/false
# _svcdiag_cfmCctimeoutMAFile: input file which contains the MA names
# _automation: Test automation - true/false
#
#------------------------------------------------------------------------------

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

### Initialize global variables ###
global trace 

### Initialize level, mac address and command history variables ###
set level ""
set mac_addr ""
set cmd ""
set script "cfm_cctimeout.tcl"
set policy 1

### check for environment variables that sets the debug flag ###

if {![info exists _debug_cfm_cctimeout]} {
    set _debug_cfm_cctimeout 0 
}
set trace $_debug_cfm_cctimeout

### Check generic policy environment variables
if {[check_policy_env "_svcdiag_cfmCctimeout" cmd automation] != 0} {
    exit
}

### Get the list of MA names from a file and exit if the list is empty ###
set ma_list {}
if { [get_ma_list $_svcdiag_cfmCctimeoutMAFile ma_list] == 0 } {
    exit_policy_script_with_errors $script
    exit
}
debug $trace "MA list is: $ma_list" 2

### Policy execution starts from here ###
debug $trace "CC timeout script starts..."

### Get triggered syslog message ###
array set event_info_array [event_reqinfo]
set _msg $event_info_array(msg)
debug $trace  "Triggered syslog message: $_msg" 2

### Parse syslog message to get mpid, vlan, MA name, domain and event code ###
parse_cfm_syslog $_msg domain MA_name vlan mpid  event

### Check whether the MA name is defined in the MA list ###
if {([match_ma $MA_name $ma_list] == 1)} {
    debug $trace "MA: $MA_name matches" 2
} else {
    debug $trace "MA: $MA_name does not match" 2
    exit
}

### Get the domain level ###
if {[parse_domain_level $domain cmd $policy level] == 0} {
    exit_policy_script_with_errors $script
    exit
} else {
    debug $trace "LEVEL: $level" 2
}

set summary ""
set details ""
set scenario "timeout"
###  Prepare Summary and Detailed Report ###
if {[prepare_report cmd summary details $mpid $domain $vlan $level $MA_name\
       $scenario] == 0} {
    exit_policy_script_with_errors $script
    exit
} 
debug $trace "Command history: $cmd, Summary:\n$summary, Details:\n$details" 2

set len [string length $summary]
debug $trace "Length of summary message is: $len"

set trigger_message $_msg
### Send out the summary and detailed message via notification ###
send_notification $summary $details $_svcdiag_cfmCctimeoutNotif\
    $_svcdiag_cfmCctimeoutConfig $_svcdiag_cfmCctimeoutEvent\
    $_svcdiag_cfmCctimeoutCommand $cmd $_svcdiag_cfmCctimeoutPrepend\
    $trigger_message $automation

debug $trace "CFM cctimeout script is done"
