::cisco::eem::event_register_syslog tag 1 pattern "\%E_CFM-6-REMOTE_MEP_UP: Continuity Check message is received from a remote MEP with mpid.*vlan.*MA name.*domain.*interface.*status.*Down.*event.*code.*PortState.*" maxrun 300

::cisco::eem::event_register_syslog tag 2 pattern "\%E_CFM-3-REMOTE_MEP_DOWN: Remote MEP mpid.*vlan.*MA.*name.*in.*domain.*changed.*state.*to.*down.*with.*event.*code.*TimeOut.*"  

       ::cisco::eem::trigger {
           ::cisco::eem::correlate event 1 or  event 2 
           ::cisco::eem::attribute tag 1 occurs 1
           ::cisco::eem::attribute tag 2 occurs 1
       } 

#------------------------------------------------------------------------------
# EEM policy to perform auto shutdown interface upon receiving syslog message  
#
# Feb 2009 - Mike Cheng (mcheng1@cisco.com)
#
# Copyright (c) 2009 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0 03/10/2009 
# Title: Shut down the local interface 
# Name: cfm_shut.tcl
# Desc: This script will shut down the local interface upon receiving remote interface down syslog message 
#
#       The structure and requirment of the file is defined below.
#       
#       For the purpose of test automation an environment variable _automation
#       is required. This environment variable is checked if defined
#       previously and if not is defined.
# 
#
# Environment Variables for this script are:
# _svcdiag_cfmShutNotif: notification [email|syslog|snmp|all] or any 
#                        combinations separated by ","
# _svcdiag_cfmShutConfig: config history [TRUE/FALSE] 
# _svcdiag_cfmShutEvent: event history [TRUE/FALSE] 
# _svcdiag_cfmShutCommand: command history [TRUE/FALSE] 
# _svcdiag_cfmShutPrepend: prepend triggered message [TRUE/FALSE] 
# _svcdiag_cfmShutMAFile: input file which contains the MA names
# _svcdiag_cfmShutP2P: P2P service for all MAs specified in the Input file
#                      [TRUE/FALSE]
# _automation: Test automation [TRUE/FALSE]
# 
#------------------------------------------------------------------------------

################################################################################
#
#    Description: check remote port status is "Up" or not
#
#    IN: mpid, domain, vlan and port status from the triggered message
#        cmd:  Command history to be updated (call by reference)
#
#    OUT: return 1 if remote port is "Up" 
#         return 0 if remote port is not "Up" 
#
################################################################################

proc is_remote_port_up {mpid domain vlan cmd} {
    global trace

    upvar $cmd cmd2

    ### get remote port status
    set remote_status [get_remote_port_status $mpid $domain $vlan cmd2]

    ### compare port status from the triggered message matches with the remote
    ### port status
    debug $trace "Remote port status: $remote_status\n"
    if {$remote_status != "Up"} {
       return 0 
    } else {
       puts "WARNING: Interface status associated with remote MEP with mpid\
             $mpid vlan $vlan MA name $domain is already Up.  Aborting CFM\
             shut script for this instance."
       return 1 
    }
}

################################################################################
#
#    Description: Check port status vs the event code
#
#    IN: vlan, ma, domain, interface_name, mpid and event code
#
#    OUT: Notification message  
#
################################################################################

proc get_shut_notification_message {vlan ma domain interface_name mpid event} {

    global trace

    ### Get domain name after trimmed 
    set domain2 [get_trimmed_domain $domain] 

    ### Setup messages
    set msg "SERV-DIAG_CFM_shut: Service failure detected on vlan $vlan, CFM MD\
              $domain2. Reason: " 

    set msg1 "Action: Interface $interface_name with local MEP $mpid has been\
              shutdown."

    if {$event == "PortState"} {
        append msg "Remote UNI failure. "
        append msg $msg1                                                      
    } elseif {$event == "TimeOut"} {
        append msg "Remote MEP timeout. "
        append msg $msg1 
    } else {
        append msg "Unknown event code. "
    }

    debug $trace "Notification Message:\n"
    debug $trace "$msg \n"
    return $msg
}


namespace import ::cisco::eem::*
namespace import ::cisco::lib::*


### Initialize Command History and global variables
set cmd ""
set policy 1
global trace 
set script "cfm_shut.tcl"

if {![info exists _debug_cfm_shut]} {
    set _debug_cfm_shut 0
}
set trace $_debug_cfm_shut

### Check generic policy environment variables                                
if {[check_policy_env "_svcdiag_cfmShut" cmd automation] != 0} { 
    exit_policy_script_with_errors $script 
    exit                                                                      
}   

### check policy specific environment variables 
if {![info exists _svcdiag_cfmShutP2P]} {
    set err "P2P service environment variable: _svcdiag_cfmShutP2P is\
             not set" 
    exit_policy_script_with_errors $script $err
    exit 
} elseif {[validate_p2p $_svcdiag_cfmShutP2P] == 0} {
    exit_policy_script_with_errors $script 
    exit
}

### Retreive MA names from a file
### If there is MA name, exit
debug $trace "MA File: $_svcdiag_cfmShutMAFile"
set ma_list {}
if {[get_ma_list $_svcdiag_cfmShutMAFile ma_list] == 0} {
    exit_policy_script_with_errors $script 
    exit    
}

debug $trace  "shutdown script starts..."  

### Retrieve triggered syslog message from EEM
array set arr_einfo [event_reqinfo]
set syslog_message $arr_einfo(msg)

debug $trace  "Triggered syslog message: $syslog_message"

# Parse syslog message for all the required information
set debug 1
parse_cfm_syslog $syslog_message cfm_domain cfm_ma cfm_vlan cfm_mpid cfm_event

### Check whether the MA name is defined is the MA list ###
if {([match_ma $cfm_ma $ma_list] == 1)} {
   debug $trace "MA: $cfm_ma is ok"
} else {
   debug $trace  "MA: $cfm_ma is ignored"
   exit
}

### Make sure remote port status is NOT "Up" 
if {[is_remote_port_up $cfm_mpid $cfm_domain $cfm_vlan cmd] == 1} { 
   ### exit if remote port is up 
   exit
}

### Get Local MEP's interface name ###
if {[get_local_mep_interface $cfm_domain $cfm_ma $cfm_vlan cfm_interface_name\
    cmd] == 0} {
   exit_policy_script_with_errors $script 
   exit
}

### Shut down the corresponing local Ethernet interface
puts  "Shutting down: $cfm_interface_name"
shut_down_interface $cfm_interface_name cmd

### Verify the local interface is down
if {[is_interface_up $cfm_interface_name cmd] == 1} {
    puts stderr "Interface is shut, but the status is not disabled"
} 

### Save the context to EEM
set context_name [format "%s-%s-%s-%s" $cfm_mpid $cfm_vlan $cfm_ma $cfm_domain]
debug $trace  "Context name: $context_name"
if {[catch {context_save $context_name "cfm*"} errmsg]} {
      debug $trace "contex not saved"
} else {
      debug $trace "contex saved"
}

### Send notification accordingly
set message [get_shut_notification_message $cfm_vlan $cfm_ma $cfm_domain $cfm_interface_name $cfm_mpid $cfm_event]

debug $trace "Cmd history:\n"
debug $trace $cmd

debug $trace $message
set long_message ""

send_notification $message $long_message $_svcdiag_cfmShutNotif $_svcdiag_cfmShutConfig $_svcdiag_cfmShutEvent $_svcdiag_cfmShutCommand $cmd $_svcdiag_cfmShutPrepend $syslog_message $automation 

debug $trace "cfm_shutdown tcl script is done"

