::cisco::eem::event_register_syslog tag 1  pattern "\%E_CFM-6-REMOTE_MEP_UP: Continuity Check message is received from a remote MEP with mpid.*vlan.*MA.*name.*domain.*interface.*status.*Up.*event.*code.*PortState.*" maxrun 300

::cisco::eem::event_register_syslog tag 2 pattern "\%E_CFM-6-REMOTE_MEP_UP: Continuity Check message is received from a remote MEP with mpid.*vlan.*MA.*name.*domain.*interface.*status.*Up.*event.*code.*New.*" 

::cisco::eem::event_register_syslog tag 3 pattern "\%E_CFM-6-REMOTE_MEP_UP: Continuity Check message is received from a remote MEP with mpid.*vlan.*MA.*name.*domain.*interface.*status.*Up.*event.*code.*Returning.*" 

::cisco::eem::event_register_syslog tag 4 pattern "\%E_CFM-6-REMOTE_MEP_UP: Continuity Check message is received from a remote MEP with mpid.*vlan.*MA.*name.*domain.*interface.*status.*AdmDn.*event.*code.*Returning.*" 

::cisco::eem::event_register_syslog tag 5 pattern "\%E_CFM-6-REMOTE_MEP_UP: Continuity Check message is received from a remote MEP with mpid.*vlan.*MA.*name.*domain.*interface.*status.*AdmDn.*event.*code.*New.*" 

       ::cisco::eem::trigger {
           ::cisco::eem::correlate event 1 or event 2 or event 3 or event 4\
                                   or event 5
           ::cisco::eem::attribute tag 1 occurs 1
           ::cisco::eem::attribute tag 2 occurs 1
           ::cisco::eem::attribute tag 3 occurs 1
           ::cisco::eem::attribute tag 4 occurs 1
           ::cisco::eem::attribute tag 5 occurs 1
       }


#------------------------------------------------------------------------------
# EEM policy to perform auto shutdown interface upon receiving syslog message  
#
# Feb 2009 - Mike Cheng (mcheng1@cisco.com)
#
# Copyright (c) 2008 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0 03/10/2008 
# Title: UnShut local interface policy script
# Name: cfm_unshut.tcl
# Desc: This script will shut down the local interface upon receiving remote
#       interface down syslog message 
#
#       The structure and requirment of the file is defined below.
#       
#       For the purpose of test automation an environment variable _automation
#       is required. This environment variable is checked if defined
#       previously and if not is defined.
# 
# Input-file:
#
# Environment Variables for this script are:
# _svcdiag_cfmUnshutNotif: notification 
#                          [email|syslog|snmp|all] or any combinations     
# _svcdiag_cfmUnshutConfig: config history [TRUE/FALSE]
# _svcdiag_cfmUnshutEvent: event history [TRUE/FALSE]  
# _svcdiag_cfmUnshutCommand: command history [TRUE/FALSE]     
# _svcdiag_cfmUnshutPrepend: prepend triggered message [TRUE/FALSE]  
# _automation: Test automation [TRUE/FALSE]  
# 
#------------------------------------------------------------------------------

################################################################################
#
#    Description: Retrieve saved shut context based on mpid, vlan, ma and domain 
#
#    IN: mpid, vlan, ma, domain 
#        interface_name: pass by reference    
#
#    OUT: return 0 if saved shut context is retrieved successful 
#         return 1 if it fails to retrieve context
#
################################################################################

proc retrieve_saved_shut_context {mpid vlan ma domain interface_name} {

    global trace

    set context_name [format "%s-%s-%s-%s" $mpid $vlan $ma $domain]
    debug $trace "Context name: $context_name"

    upvar $interface_name interface_name2

    if {[catch {foreach {var value} [context_retrieve $context_name] {set $var $value}} errmsg]} {
        debug $trace "context_retrieve failed: $errmsg"
        return 1 
    } else {
        debug $trace "context_retrieve succeeded"
    }

    if {[info exists cfm_interface_name]} {
        debug $trace "cfm_interface_name exists and is $cfm_interface_name"
        set interface_name2 $cfm_interface_name
    } else { 
        puts "cfm_interface_name does not exist"
        return 1 
    } 
    return 0

}

################################################################################
#
#    Description: Check port status vs the event code
#
#    IN: vlan, ma, domain, interface_name, mpid and event code
#
#    OUT: Notification message
#
################################################################################

proc get_unshut_notification_message {vlan ma domain interface_name mpid event} {

    global trace

    ### Get domain name after trimmed
    set domain2 [get_trimmed_domain $domain] 

    set msg "SERV-DIAG_CFM_unshut: Service recovery detected on vlan $vlan,\
             CFM MD $domain2. Reason: "
    set msg1 "Action: Interface $interface_name with local MEP mpid $mpid has\
              been unshut."
    if {$event == "PortState"} {
        append msg "Remote UNI up. "
        append msg $msg1
    } elseif {($event == "New") || ($event == "Returning")} {
        append msg "Remote MEP reachable. "
        append msg $msg1
    } else {
        append msg "Unknown event code"
    }
    debug $trace "Notification Message:\n"
    debug $trace "$msg \n"
    return $msg
}

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

### Initialize Command History and global variables
set cmd ""
set policy 1
global trace
set script "cfm_unshut.tcl"

if {![info exists _debug_cfm_unshut]} {
    set _debug_cfm_unshut 0
}
set trace $_debug_cfm_unshut

### Check generic policy environment variables
if {[check_policy_env "_svcdiag_cfmUnshut" cmd automation] != 0} {
    exit_policy_script_with_errors $script
    exit
}

debug $trace "Unshut script starts..."

### Retrieve triggered syslog message from EEM
array set arr_einfo [event_reqinfo]
set syslog_message $arr_einfo(msg)

debug $trace  "Triggered syslog message: $syslog_message"

# Parse syslog message for all the required information
parse_cfm_syslog $syslog_message domain ma vlan mpid event_code

### Retrieve the context from EEM
if {[retrieve_saved_shut_context $mpid $vlan $ma $domain interface_name] == 1} {
    exit 
}

### ensure that local interface matches with the DB query
if {[is_valid_interface $interface_name cmd] != 1} {
    set err "Invalid Interface $interface_name can't be un-shut\
           by the script"
    exit_policy_script_with_errors $script $err 
    exit
}

### No shut the interface
puts "No shut on $interface_name ..."
no_shut_interface $interface_name cmd

### Verify the local interface is up 
if {[is_interface_up $interface_name cmd] == 0 } {
    set err "Interface is unshut, but the status is down"
    exit_policy_script_with_errors $script $err 
    exit
} else {
    debug $trace "Interface is up..."
}

### Send notification accordingly
set message [get_unshut_notification_message $vlan $ma $domain $interface_name $mpid $event_code]

debug $trace "Cmd history:\n"
debug $trace $cmd
set long_message "" 

send_notification $message $long_message $_svcdiag_cfmUnshutNotif $_svcdiag_cfmUnshutConfig $_svcdiag_cfmUnshutEvent $_svcdiag_cfmUnshutCommand $cmd $_svcdiag_cfmUnshutPrepend $syslog_message $automation 

debug $trace "Unshut script is done"
