#  ===================================================================
#  Ciso IOS TCL                                      ||        ||
#                                                    ||        ||
#  CFM On-demand                                    ||||      ||||
#                                               ..:||||||:..:||||||:..
#                                             ------------------------
#                                              C i s c o  S y s t e m s 
#   ===================================================================
#
# Wrapper script for CFM On-Demand Scenario registration script 
#
# Jun 2009 - Greg Thompson (gst@cisco.com)
#            Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007-2009 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-06/19/2009
# Title: Wrapper script for On-Demand Scenario  
#
# Name: cfmod.tcl
#
# Desc: Facilitates minimal argument inputs as CLI to modify MA name, Domain 
#       name AND optional Remote Mep-id & notification method given for the CFM
#       On-Demand Scenario 
#
# Usage: tclsh cmfod.tcl ma_name domain_name
#
# optional args
#
# [-m mep_id] - MEP ID 
#      default: NULL
#
# [-n email | syslog | snmp | all]  (combine using CSV, e.g. email,syslog)
#      default: syslog
#
#=============================================================================

namespace eval cfmod {
    proc PrintUsageInfo {} {
        if {$::tcl_interactive == 1} {
            set a1 cfmod
        } else {
            set a1 "tclsh cfmod.tcl"
        }
        puts -nonewline "usage: $a1"
        puts { ma_name domain_name [-m mep_id] [-n email | syslog | snmp | all]}
    }

    proc get_eem_dirs {pol lib} {
        upvar $pol svc_dir
        upvar $lib user_lib
        set svc_dir [exec show event man dir user policy]
        if {[regexp "not configured" $svc_dir match]} {
             unset svc_dir
             puts stderr "Cannot determine EEM policy directory!\n\nPlease\
                          either configure EEM user policy directory, or\
                          provide the URL using the -u option."
             return 0
        }
        set user_lib [exec show event man dir user lib]
        if {[regexp "not configured" $user_lib match]} {
             unset user_lib
             puts stderr "Cannot determine EEM user library directory!\n\n\
                          Please either configure the EEM user library\
                          directory, or provide the URL using the -l option."
             return 0
        }
        return 1
    }

    proc checkIfPolicyRegistered {} {
        set output [exec show event manager policy registered]
        if {[regexp "cfm_ondemand.tcl" $output] == 0} {
            puts "ERROR: The policy cfm_ondemand.tcl should be registered first\
                  to make use of the wrapper script." 
            return 0
        } else {
            return 1
        } 
    }

    proc validate_notify {notif2 lib_url} {
        set err 0
        set email 0
        set snmp 0
        set syslog 0
        set err "\n\nERROR: Invalid notification method:"

        set notif [string tolower $notif2]
            if {($notif == "email")} {
                set email 1
            } elseif {($notif == "all")} {
                set email 1
                set snmp 1
                set syslog 1
            } elseif { ($notif == "snmp") } {
                set snmp 1
            } elseif {($notif == "syslog")} {
                set syslog 1
            } elseif {[string first "," $notif] > 0} {
                foreach token [split $notif ","] {
                    set token2 [string trim $token]
                    if {($token2 == "email")} {
                        if {$email == 1} {
                            puts "\n\nERROR: Duplicate \"email\" notification\
                                   method"
                            return 0
                        }
                        set email 1
                    } elseif {($token2 == "syslog")} {
                        if {$syslog == 1} {
                            puts "\n\nERROR: Duplicate \"syslog\" notification\
                                  method"
                            return 0
                        }
                        set syslog 1
                    } elseif {($token2 == "snmp")} {
                        if {$snmp == 1} {
                            puts "\n\nERROR: Duplicate \"snmp\" notification\
                                  method"
                            return 0
                        }
                            set snmp 1
                    } else {
                        puts "$err $token2"
                        return 0
                    }
                }
            } else {
                puts "$err $notif"
                return 0
            }

        if {($email == 0) && ($snmp == 0) && ($syslog == 0)} {
            puts "$err $notif"
            return 0
        }

        set flag 1
        if {$email == 1} {
            if {[file exists $lib_url/userlib.tcl] == 0} {
                puts "Error: Cannot find userlib.tcl in $lib_url. \
                      Please re-install Service Diagnostics CFM libraries."
                return 0
            } else {
                source $lib_url/userlib.tcl
                if {[emailInputAlreadyGiven "email"] != "configured"} {
                    puts "Run the file 'collectEmailParameters.tcl'"
                    puts "Usage: tclsh\
                          disk#:/dir_name/collectEmailParameters.tcl\
                          email_fromID email_toID email_ ccID\
                          email_server_name\n\n"
                    set flag 0
                }
            }
        }

        if {$snmp == 1} {
            set policy 0
            if {[check_snmp_trap_setting $policy] != 0} {
                set flag 0 
            }
        }

        return $flag 
    }
 
    proc cfmod { {ma_name ""} {domain_name ""} args } {
        if {[::cfmod::checkIfPolicyRegistered] == 1} {
            set notification syslog
            set pol_url ""
            set lib_url ""
            set mep_id "NULL"
            set state flag
        } else {
            return
        }

        if {([string length $ma_name] == 0) || ([string length $domain_name]\
              == 0)} {
            ::cfmod::PrintUsageInfo
            return
        }

        foreach arg $args {
            switch -- $state {
                flag {
                    switch -glob -- $arg {
                        \-m {
                           set state mep_value
                       }
                        \-n {
                           set state not_value
                       }
                        \-u {
                           set state pol_value
                       }
                        \-l {
                           set state lib_value
                       }
                       default {
                           ::cfmod::PrintUsageInfo
                           return
                       }
                   }
               }
               mep_value {
                   set mep_id $arg
                   set state flag
               }
               not_value {
                   set notification $arg
                   set state flag
               }
               pol_value {
                   set pol_url $arg
                   set state flag
               }
               lib_value {
                   set lib_url $arg
                   set state flag
               }
            }
        }

        if {[string length $pol_url] == 0 || [string length $lib_url] == 0} {
            if { [get_eem_dirs pol_url lib_url] == 0 } {
                return
            }
        }

        if {[file exists $lib_url/cfm_lib.tcl] == 0} {
            puts "Error: Cannot find cfm_lib.tcl in $lib_url. Please \
                  re-install Service Diagnostics CFM libraries."
        } else {
            source $lib_url/cfm_lib.tcl
        }
 
        if {[file exists $lib_url/diag_lib.tcl] == 0} {
            puts "Error: Cannot find diag_lib.tcl in $lib_url. \
                  Please re-install Service Diagnostics CFM libraries."
        } else {
           source $lib_url/diag_lib.tcl
        }
        
        if {[file exists $lib_url/userlib_2.tcl] == 0} {
            puts "Error: Cannot find userlib_2.tcl in $lib_url. \
                  Please re-install Service Diagnostics CFM libraries."
        } else {
           source $lib_url/userlib_2.tcl
        }

        set cmd "" 
        set policy 0

        ### validate domain ###
        if {[validate_domain $domain_name $cmd $policy] == 0} {
            return 0
        }

        ### validate ma name ###
        if {[validate_ma_name $domain_name $ma_name $policy] == 0} {
            return 0
        }

        ### validate MEP id and modify environment variable ###
        if {[string length $mep_id] == 0} {
            set mep_id "NULL"
        } else { 
            if {[validate_mpid $mep_id cmd $domain_name $policy $ma_name]\
                == 0} {
                return 0
            }
        }

        ### validate notification method and modify environment variable ###
        if {[validate_notify $notification $lib_url] == 0} {
            return 0
        }

        ### modify the EEM environment variable ###
        set_eem_env _svcdiag_cfmOndemandMAname $ma_name
        set_eem_env _svcdiag_cfmOndemandDomain $domain_name
        set_eem_env _svcdiag_cfmOndemandMpid $mep_id
        set_eem_env _svcdiag_cfmOndemandNotif $notification

        return 1
       
    }
    namespace export cfmod
}

namespace import ::cfmod::cfmod

global trace
### Set the debug for helper scripts ###
if {![info exists _debug_cfm_deploy]} {
     set _debug_cfm_deploy 0
}
set trace $_debug_cfm_deploy

if {$tcl_interactive == 0} {
    if {$argc < 2} {
        ::cfmod::PrintUsageInfo
    } else {
        set temp [eval "cfmod $::argv"]
        if {$temp == 1} {
            exec "event manager run cfm_ondemand.tcl type user"
        } else {
            return
        }
    }
}

#Cisco Tcl Signature V1.0
#3082077506092a864886f70d010702a082076630820762020101310b3009
#06052b0e03021a0500300b06092a864886f70d010701a08204bb308204b7
#3082039fa003020102020900d48b1bbfb069e9b9300d06092a864886f70d
#0101050500308198310b3009060355040613024341311230100603550408
#130963616c69666f6e69613111300f060355040713086d69706c69746173
#311b3019060355040a1312636973636f2073797374656d7320696e632e31
#0e300c060355040b13054e53535447311630140603550403130d43687269
#7374696e652059616e311d301b06092a864886f70d010901160e6379616e
#40636973636f2e636f6d301e170d3037313231383138333232375a170d31
#30313231373138333232375a308198310b30090603550406130243413112
#30100603550408130963616c69666f6e69613111300f060355040713086d
#69706c69746173311b3019060355040a1312636973636f2073797374656d
#7320696e632e310e300c060355040b13054e535354473116301406035504
#03130d436872697374696e652059616e311d301b06092a864886f70d0109
#01160e6379616e40636973636f2e636f6d30820122300d06092a864886f7
#0d01010105000382010f003082010a02820101009c22f1d01af397756ab6
#cfce92a9e761b73fbf1efd070aa4e90448e6077e2c127ff47a5951de7790
#55da014ff152576a08d7199b8ad5757eb8c9a7c1a493715df2176e9f56f0
#74735d43420cf1ed17fe0aa3494724808aefcf274572c7750be071a94560
#9c045c48a5454f9e7933bf93d967bcf582918b1bbd48f47dd978c7ea9c88
#23eb01482a40ef770f823ea2e63eaf7cab5367bd51b54979925464e27b22
#82fc5bfe94f6f41f2f91e56444ae371a050107fe79fd5db3f228bbbd5fcb
#653a87a3c1268d74500ba1489382290804d9d7937d3269b1c55eae51f9f7
#62eadbd4226ba6d4008414b353181ee6cfbe24204e4a82436b117c255014
#77ec74c3ac4d0203010001a38201003081fd301d0603551d0e041604149c
#0e12d3a222891fc1300b780f38b54e07e228073081cd0603551d230481c5
#3081c280149c0e12d3a222891fc1300b780f38b54e07e22807a1819ea481
#9b308198310b300906035504061302434131123010060355040813096361
#6c69666f6e69613111300f060355040713086d69706c69746173311b3019
#060355040a1312636973636f2073797374656d7320696e632e310e300c06
#0355040b13054e53535447311630140603550403130d436872697374696e
#652059616e311d301b06092a864886f70d010901160e6379616e40636973
#636f2e636f6d820900d48b1bbfb069e9b9300c0603551d13040530030101
#ff300d06092a864886f70d010105050003820101003e6d657e30c21acb30
#d7e91aef99cd6f83a8644d7ae437fb4a0cd1a8c6a0f1cb2194be80868b75
#1d20ee8dc72a1f80de697928820ec3f806cec67df136d6f8b6ff0ba7ef6f
#e4c228b37e7407becff9903198ebe2258d3774f198de8d0366cb10496408
#57dff62f247dfcae9813fde4167c1f959b291490cfb8e0b8db3395ce019f
#07c94aa4a86d1a0dae2b6cc6d139db740841dc65add58512ea59336dbc8c
#9abc14d500e42c8ea5031c201eec027d14e9878974629d2fdbe0d0cfffab
#8ab73f148bdec788de59550443ccf12cb3e74518274c3d7d2892eeb90f23
#0d5e853b85542c05b12198c097c2fb4023b370f0629645e11552f5429fb8
#aacef60a9344b5318202823082027e0201013081a6308198310b30090603
#55040613024341311230100603550408130963616c69666f6e6961311130
#0f060355040713086d69706c69746173311b3019060355040a1312636973
#636f2073797374656d7320696e632e310e300c060355040b13054e535354
#47311630140603550403130d436872697374696e652059616e311d301b06
#092a864886f70d010901160e6379616e40636973636f2e636f6d020900d4
#8b1bbfb069e9b9300906052b0e03021a0500a081b1301806092a864886f7
#0d010903310b06092a864886f70d010701301c06092a864886f70d010905
#310f170d3039303932353139353833325a302306092a864886f70d010904
#31160414e6b376e7a7004fd849852f4954d98cf62bf6747b305206092a86
#4886f70d01090f31453043300a06082a864886f70d0307300e06082a8648
#86f70d030202020080300d06082a864886f70d0302020140300706052b0e
#030207300d06082a864886f70d0302020128300d06092a864886f70d0101
#010500048201008411738b349cb4cdb8da36992b89c31596fe9e6518ad95
#5efd60323b8033d5e1d1ced009c5e4e48fd1d26dc1bab023ca5cd65895f8
#e71c9e6b416d5b857f7906122dd703ebe4a760a9458bfa3bc34927431964
#d332cd2382452107232634758e163be7e89884a619cff5db9d37b3294aff
#308568121c0229571c71e4f59b1b3876df0d40d2084701d9a4a0a652f6a3
#abcf8a422d082f7f0e1c0773304c5f5fae40ff9dc7e36193875d5c56e589
#3a61b53d91e400a2d025bbbba7f6c195947309dd8918ae79c96b88b6e6e1
#88e72b80d4de33bec67b5ac7781fd71b968f63b802971cd651a1241e6f40
#8661e96db83aa5d2dd5e0730cdbbfa3eb431695c965b35
