
#------------------------------------------------------------------------------
# Diagnostic script to get the inputs related to "CFM",
#     "CFM Shut, Unshut, Auto Trace, CC Timeout and On Demand" 
#
# Feb 2009 -  Vimaladevi Rajakanthan (vrajakan@cisco.com)
#             Mike Cheng (mcheng1@cisco.com) 
#
# Copyright (c) 2009 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-03/17/2009
# Title: Store(configure) inputs from user and register with EEM.
# 
# Name: sdiag_cfm.tcl
#
# Desc: Gets input from customer, validates them and and stores(configures on
#       the router) the inputs. Registers the corresponding CFM policies
#       "cfm_shut.tcl", "cfm_unshut.tcl" and "cfm_autotrace.tcl" with EEM. 
#
#       The inputs will be collected from user non-interactively
#
# Usage: tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl <cfm policy name> <notification>
#        <configurationHistory> <eventHistory> <commandHistory> 
#        <prepend> <userPolicyDirectory> <userLibraryDirectory> 
#        <additionalInputs> where
#
#        The <cfm policy name> -- ["cfm_ondemand.tcl" or "cfm_cctimeout.tcl" 
#                                  or "cfm_autotrace.tcl" or "cfm_shut.tcl"
#                                  or "cfm_unshut.tcl"]
#            <notification> -- ["email" or "syslog" or "snmp" or "all"] 
#                           -- ["email,syslog",.., "email,syslog,snmp"]
#            <configurationHistory> -- ["TRUE" or "FALSE"]
#            <eventHistory>  -- ["TRUE" or "FALSE"]
#            <commandHistory>  -- ["TRUE" or "FALSE"]
#            <Prepend>  -- ["TRUE" or "FALSE"]
#            <userPolicyDirectory> -- [absolute path of policy directory]
#            <userLibraryDirectory> -- [absolute path of library directory]
#            <additionalInputs> -- depends on the specified policy
#        
#-----------------------------------------------------------------------------        
#############################################################################
#
#    Description: Checks if the user library and policy directory is 
#                 configured and registered with EEM
#
#    IN :None
#    OUT:sources the files userlib.tcl,userlib_2.tcl,diag_lib.tcl
#                          cfm_lib.tcl
#
#############################################################################

proc sourceLibFile {user_lib user_policy} {

    ### validate user library path 
    set error 0
    set temp [file join  $user_lib cfm_lib.tcl]
    if {[file exists $temp] != 0} {
        set user_lib_dir $user_lib
    } else {
        puts "\nERROR: User library directory path \"$user_lib\"\
              is not valid."
        set error 1 
    }

    ### validate user policy path 
    set temp [file join  $user_policy sdiag_cfm.tcl]
    if {[file exists $temp] != 0} {
        set user_policy_dir $user_lib
    } else {
        puts "\nERROR: User policy directory path \"$user_policy\"\
             is not valid."
        set error 1 
        
    }

    ### return error if one of the paths is not correct
    if {$error == 1} {
        return 0
    }

    source [file join $user_lib_dir diag_lib.tcl]
    source [file join $user_lib_dir userlib.tcl]
    source [file join $user_lib_dir userlib_2.tcl]
    source [file join $user_lib_dir cfm_lib.tcl]

    return 1
}

#############################################################################
#
#    Description: Configure EEM environment variables for CFM OnDemand 
#                 scenario 
#
#    IN: The inputs given by the user  
#    OUT:  Configures the inputs as  environment variables on the router 
#      
###############################################################################

proc ondemand_config {notif config event command prepend ma_name\
                      domain mpid} {
    
    set_eem_env _svcdiag_cfmOndemandNotif $notif
    set_eem_env _svcdiag_cfmOndemandConfig $config
    set_eem_env _svcdiag_cfmOndemandEvent $event
    set_eem_env _svcdiag_cfmOndemandCommand $command
    set_eem_env _svcdiag_cfmOndemandPrepend $prepend
    set_eem_env _svcdiag_cfmOndemandMAname $ma_name
    set_eem_env _svcdiag_cfmOndemandDomain $domain
    set_eem_env _svcdiag_cfmOndemandMpid $mpid
    set code [reg_eem_policy cfm_ondemand.tcl]
    return $code
}

#############################################################################
#
#    Description: Configure EEM environment variables for CFM CCTimeout 
#                 scenario 
#
#    IN: The inputs given by the user  
#    OUT:  Configures the inputs as  environment variables on the router 
#      
###############################################################################

proc cctimeout_config {notif config event command prepend ma_file} {
    
    set_eem_env _svcdiag_cfmCctimeoutNotif $notif
    set_eem_env _svcdiag_cfmCctimeoutConfig $config
    set_eem_env _svcdiag_cfmCctimeoutEvent $event
    set_eem_env _svcdiag_cfmCctimeoutCommand $command
    set_eem_env _svcdiag_cfmCctimeoutPrepend $prepend
    set_eem_env _svcdiag_cfmCctimeoutMAFile $ma_file
    set code [reg_eem_policy cfm_cctimeout.tcl]
    return $code
}

#############################################################################
#
#    Description: Configure EEM environment variables for CFM shut 
#                 scenario 
#
#    IN: The inputs given by the user  
#    OUT:  Configures the inputs as  environment variables on the router 
#      
###############################################################################

proc shut_config {notif config event command prepend ma_file\
                  p2p_service} {
    
    set_eem_env _svcdiag_cfmShutNotif $notif
    set_eem_env _svcdiag_cfmShutConfig $config
    set_eem_env _svcdiag_cfmShutEvent $event
    set_eem_env _svcdiag_cfmShutCommand $command
    set_eem_env _svcdiag_cfmShutPrepend $prepend
    set_eem_env _svcdiag_cfmShutMAFile $ma_file
    set_eem_env _svcdiag_cfmShutP2P $p2p_service
    set code [reg_eem_policy cfm_shut.tcl]
    return $code
}

#############################################################################
#
#    Description: Configure EEM environment variables for CFM unshut 
#                 scenario 
#
#    IN: The inputs given by the user  
#    OUT:  Configures the inputs as  environment variables on the router 
#      
################################################################################

proc unshut_config {notif config event command prepend} {
    
    set_eem_env _svcdiag_cfmUnshutNotif $notif
    set_eem_env _svcdiag_cfmUnshutConfig $config
    set_eem_env _svcdiag_cfmUnshutEvent $event
    set_eem_env _svcdiag_cfmUnshutCommand $command
    set_eem_env _svcdiag_cfmUnshutPrepend $prepend
    set code [reg_eem_policy cfm_unshut.tcl]
    return $code
}

#############################################################################
#
#    Description: Configure EEM environment variables for CFM auto trace 
#                 scenario
#
#    IN: The inputs given by the user
#    OUT:  Configures the inputs as  environment variables on the router
#
#############################################################################                 
proc autotrace_config {notif config event command prepend ma_file\
                       time} {

    set_eem_env _svcdiag_cfmAutotraceNotif $notif
    set_eem_env _svcdiag_cfmAutotraceConfig $config
    set_eem_env _svcdiag_cfmAutotraceEvent $event
    set_eem_env _svcdiag_cfmAutotraceCommand $command
    set_eem_env _svcdiag_cfmAutotracePrepend $prepend
    set_eem_env _svcdiag_cfmAutotraceMAFile $ma_file
    set_eem_env _svcdiag_cfmAutotraceTimer $time
    set code [reg_eem_policy cfm_autotrace.tcl]
    return $code
}

proc exit_script_with_errors {script {message ""} } {

    if {[string length $message] != 0} {
        puts "\nERROR: $message"
    }
    set verbose 1 
    puts [get_cfm_usage $script $verbose ]
    puts "\n\n"
}

#############################################################################
#
#    Description: Display MA warning message for cfm shut script 
#
#    IN: None
# 
#    OUT: None 
#
################################################################################

proc display_shut_ma_warning_message { } {

    set message [get_shut_ma_warning_message]
    puts "\nWARNING: $message \n"
}

################################################################################
#
#    Description: get MA warning message for shut script
#
#    IN: None
#
#    OUT: warning message
#
################################################################################

proc get_shut_ma_warning_message {} {

    set message "User should specify MAs associated to P2P services with UP\
                 MEPs on local interface without service multiplexing.\
                 (i.e. there is only one service/MEP per interface).\
                 Otherwise the action of the shut script will impact other\
                 services."

    return $message
}

################################################################################
#
#    Description: get MA warning message for shut script
#
#    IN: None
#
#    OUT: warning message
#
################################################################################

proc get_shut_help_ma_warning_message {} {

    set message "\[TRUE\] if specified MAs associated to P2P services with UP\
                 MEPs on local interface without service multiplexing.\
                 (i.e. there is only one service/MEP per interface).\
                 \[FALSE\] if above criteria is not met." 
    return $message
}

################################################################################
#
#    Description: Display CFM policy variables 
#
#    IN:  script: name of the policy 
#
#    OUT:  return policy variables 
#
################################################################################

proc display_cfm_variable {script} {

    set result [exec "show event manager environ"]

    if {$script == "cfm_shut.tcl"} {
        set prefix "_svcdiag_cfmShut"
    } elseif {$script == "cfm_unshut.tcl"} {  
        set prefix "_svcdiag_cfmUnshut" 
    } elseif {$script == "cfm_autotrace.tcl"} {  
        set prefix "_svcdiag_cfmAutotrace" 
    } elseif {$script == "cfm_cctimeout.tcl"} {  
        set prefix "_svcdiag_cfmCctimeout" 
    } elseif {$script == "cfm_ondemand.tcl"} {  
        set prefix "_svcdiag_cfmOndemand" 
    }

    puts "\n\n" 
    puts "$script Environment Variable(s):"
    set i 0
    foreach line [split $result "\n"] {
        if {[regexp {_svcdiag_(\S*)(\s*)(.*)} $line match temp temp2 \
             temp3] == 1} {
           if {[string first $prefix $line] >= 0} {
               incr i
               puts "$i. _svcdiag_$temp = $temp3"
           } 
        } 
    }
    if {$i == 0} {
        puts "There are no $script environment variables" 
    }
    puts "\n\n"

}

################################################################################
#
#    Description: get CFM Common policy usages
#
#    IN:  None
#
#    OUT:  return usage message
#
################################################################################

proc get_cfm_common_usage {} {

    set msg "\n\n<notification> \[snmp | syslog | email | all\]\
             or \[snmp,syslog\ | email,syslog | snmp,email | ...\]"
    append msg "\n<configuration history> \[TRUE | FALSE\]"
    append msg "\n<event history> \[TRUE | FALSE\]"
    append msg "\n<command history> \[TRUE | FALSE\]"
    append msg "\n<prepend trigger message> \[TRUE | FALSE\]"
    append msg "\n<user_pol_dir> full path for user policy directory"
    append msg "\n<user_lib_dir> full path for user library directory"

    return $msg
}

################################################################################
#
#    Description: get CFM policy usages
#
#    IN:  script: name of the policy script
#         verbose flag: [TRUE/FALSE]
#
#    OUT:  return usage message
#
################################################################################

proc get_cfm_usage {script verbose} {

    if {$script == "cfm_shut.tcl"} {
        return [get_cfm_shut_usage $verbose]
    } elseif {$script == "cfm_unshut.tcl"} {
        return [get_cfm_unshut_usage $verbose]
    } elseif {$script == "cfm_autotrace.tcl"} {
        return [get_cfm_autotrace_usage $verbose]
    } elseif {$script == "cfm_ondemand.tcl"} {
        return [get_cfm_ondemand_usage $verbose]
    } elseif {$script == "cfm_cctimeout.tcl"} {
        return [get_cfm_cctimeout_usage $verbose]
    } elseif {$script == "all"} {
        append msg [get_cfm_cctimeout_usage $verbose]
        append msg [get_cfm_ondemand_usage $verbose]
        append msg [get_cfm_shut_usage $verbose]
        append msg [get_cfm_unshut_usage $verbose]
        append msg [get_cfm_autotrace_usage $verbose]
        append msg "\n\nFor policy detailed parameters, use <policy> -help"
        append msg "\nFor example: tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl\
                    cfm_shut.tcl -help"
        append msg "\n\nFor policy environment variables, use <policy>\
                    -variables"
        append msg "\nFor example: tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl\
                    cfm_shut.tcl -variables"
    }
}

################################################################################
#
#    Description: get CFM shut policy usages
#    
#    IN:   verbose flag: [TRUE/FALSE]
#
#    OUT:  return usage message
#
################################################################################

proc get_cfm_shut_usage {help} {

    append err "\n\nCFM SHUT POLICY:"
    append err "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl\
                cfm_shut.tcl <notification> <configuration history>\
                <event history> <command history> <prepend trigger message>\
                <user_pol_dir> <user_lib_dir> <ma_list> <p2p>"
    if {$help == 2} {
        append err [get_cfm_common_usage]
        append err "\n<ma_list> full path file name containing one MA\
                      per line.  Wild card \"*\" can be used for all MAs\
                      configured in IOS"
        append err "\n<p2p> [get_shut_help_ma_warning_message]"
        append err "\n"
    } elseif {$help == 1} {
        append err "\n"
        append err "\nFor policy detailed parameters, use:\
                   tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl cfm_shut.tcl -help" 
    }
    return $err
}

################################################################################
#    
#    Description: get CFM unshut policy usages
#    
#    IN:   verbose flag: [TRUE/FALSE]
#    
#    OUT:  return usage message
#    
################################################################################

proc get_cfm_unshut_usage {help} {

    append error "\n\nCFM UNSHUT POLICY:"
    append error "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl\
                  cfm_unshut.tcl <notification> <configuration history>\
                  <event history> <command history> <prepend trigger message>\
                  <user_pol_dir> <user_lib_dir>"
    if {$help == 2} {
        append error [get_cfm_common_usage]
    } elseif {$help == 1} {                                                     
        append error "\n"                                                         
        append error "\nFor policy detailed parameters:\
                   tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl cfm_unshut.tcl -help"
    }    
    return $error
}


################################################################################
#    
#    Description: get CFM autotrace policy usages
#    
#    IN:   verbose flag: [TRUE/FALSE]
#    
#    OUT:  return usage message
#    
################################################################################

proc get_cfm_autotrace_usage {help} {

    append error "\n\nCFM AUTO-TRACE POLICY:"
    append error "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl\
                  cfm_autotrace.tcl <notification> <configuration history>\
                  <event history> <commandHistory> <prepend trigger message>\
                  <user_pol_dir> <user_lib_dir> <ma_list> <timer>"
    if {$help == 2} {
        append error [get_cfm_common_usage]
        append error "\n<ma_list> full path file name containing one MA\
                      per line.  Wild card \"*\" can be used for all MAs\
                      configured in IOS"
        append error "\n<timer> timer is in minutes.  Default is 6 hours.\
                      Minimum value is 1 hour."
        append error "\n"
    } elseif {$help == 1} { 
        append error "\n\nFor policy detailed parameters:\
                       tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl\
                       cfm_autotrace.tcl -help"
    }  
    return $error
}

################################################################################
#
#    Description: get CFM Ondemand policy usage
#
#    IN:   verbose flag which is [TRUE/FALSE]
#
#    OUT:  return usage message
#
################################################################################

proc get_cfm_ondemand_usage {help} {

    append error "\n\nCFM ON-DEMAND POLICY:"
    append error "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl\
                  cfm_ondemand.tcl <notification> <configurationHistory>\
                  <EventHistory> <CommandHistory> <prepend trigger message>\
                  <user_pol_dir> <user_lib_dir> <domain_name> <ma_name>\
                  <optional mep-id>"
    if {$help == 2} {
        append error [get_cfm_common_usage]
        append error "\n<domain_name> is a valid domain name under which the\
                      above specified MA is defined"
        append error "\n<ma_name> is a valid MA name under the above specified\
                      domain, for which details needs to be collected."
        append error "\n<optional mep-id> is MEP-ID for the above specified\
                      MA. This input is optional."
        append error "\n"
    } else {
        if { $help == 1 } {
            append error "\n\nFor policy detailed parameters:\
                          tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl\
                          cfm_ondemand.tcl -help"
        }
    }
    return $error
}

################################################################################
#
#    Description: get CFM CC-timeout policy usage
#
#    IN:   verbose flag which is [TRUE/FALSE]
#
#    OUT:  return usage message
#
################################################################################

proc get_cfm_cctimeout_usage {help} {

    append error "\n\nCFM CC-TIMEOUT POLICY:"
    append error "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl\
                   cfm_cctimeout.tcl <notification> <configurationHistory>\
                   <EventHistory> <CommandHistory> <prepend trigger message>\
                   <user_pol_dir> <user_lib_dir> <ma_list>"
    if {$help == 2} {
        append error [get_cfm_common_usage]
        append error "\n<ma_list> full path file name where the file has\
                       one MA per line.  Wild card \"*\" can be used for all\
                       MAs configured in IOS"
        append error "\n"
    } else {
        if {$help == 1} { 
            append error "\n\nFor policy detailed parameters:\
                          tclsh <disk#:>/<dir_name>/sdiag_cfm.tcl\
                          cfm_timeout.tcl -help"
        }
    }  
    return $error
}

################################################################################
#
#    Description: Validate the policy name 
#
#    IN: policy: policy name 
#
#    OUT:  return 1 if is successful.  Otherwise, return 0 
#
################################################################################

proc is_valid_policy {policy} {

    if {$policy == "cfm_shut.tcl"} {
        return 1
    } elseif {$policy == "cfm_unshut.tcl"} {
        return 1
    } elseif {$policy == "cfm_autotrace.tcl"} {
        return 1
    } elseif {$policy == "cfm_cctimeout.tcl"} {
        return 1
    } elseif {$policy == "cfm_ondemand.tcl"} {
        return 1
    } else {
        return 0
    }
}

################################################################################
#
#    Description: Validate the policy name 
#
#    IN: key name 
#
#    OUT:  return 1 if the input key matches "-help".  Otherwise, return 0 
#
################################################################################

proc is_valid_help_key {key} {

    if {$key == "-help"} {
        return 1
    }
    return 0
}

#############################################################################
#
#    Description: Procedure to display error message when there is no input
#                 arguments or if there are incorrect number of arugments 
#
#    IN: argument count, error string and policy script 
#
#    OUT: None 
#
###############################################################################

proc incorrect_arguments_error {argc script} {

    puts "\n\nERROR: Incorrect number of arguments"
    set verbose 1 
    puts [get_cfm_usage $script $verbose] 
}

################################################################################
#
#    Description: Display usage accordingly based on the policy name 
#
#    IN: policy name 
#
#    OUT:  Display usage accordingly based on the policy name 
#
################################################################################

proc display_usage {policy} {

    if {[is_valid_policy $policy] == 1} {
       puts [get_cfm_usage $policy 1]
    } else {
        ### else display usage for all
        puts [get_cfm_usage all 0]
    }

}

#############################################################################
#
#    Description: Validates the inputs given for additional diagnoses
#
#    IN: A string which contains the values "TRUE" or "FALSE" or some other
#        value
#    OUT: returns 1 if the input string is not "TRUE" or "FALSE"
#         otherwise returns 0
#
#############################################################################

proc check_flag {value} {

    if {($value != "TRUE") && ($value != "FALSE")} {
        return 1
    } else {
        return 0
    }
}

#############################################################################
#
#    Description: Validate common flags 
#
#    IN: config, event, command, prepend flag  
#        
#    OUT: returns 1 if the input string are ok.
#         otherwise returns 0
#
#############################################################################

proc validate_common_flags {config event command prepend} {

    set code 1 
    if {([check_flag $config] == 1) || ([check_flag $event] == 1) ||
        ([check_flag $command] == 1) || ([check_flag $prepend] == 1) } {
        puts "\n\n"
        puts "The value you entered for configurationHistory: is  \"$config\""
        puts "The value you entered for eventHistory is:  \"$event\""
        puts "The value you entered for commandHistory is:  \"$command\""
        puts "The value you entered for prepend is:  \"$prepend\""
        puts "Enter either 'TRUE' or 'FALSE' as input"
        set code 0 
    }

    return $code
}

#############################################################################
#    Description: validate the notification methods and corresponding
#                 configuration
#
#    IN:   notif - notification method
#    OUT:  return 1 if validation is successful.  Otherwise return 0 
#
##############################################################################

proc validate_notify {notif2} {

    set err 0
    set email 0
    set snmp 0
    set syslog 0
    set err "\n\nERROR: Invalid notification method:"

    set notif [string tolower $notif2]
    if {($notif == "email")} {
        set email 1
    } elseif {($notif == "all")} {
        set email 1
        set snmp 1
        set syslog 1
    } elseif { ($notif == "snmp") } {
        set snmp 1
    } elseif {($notif == "syslog")} {
        set syslog 1
    } elseif {[string first "," $notif] > 0} {
        foreach token [split $notif ","] {
            set token2 [string trim $token]
            if {($token2 == "email")} {
               if {$email == 1} {
                   puts "\n\nERROR: Duplicate \"email\" notification method"
                   return 0
               }
               set email 1
            } elseif {($token2 == "syslog")} {
               if {$syslog == 1} {
                   puts "\n\nERROR: Duplicate \"syslog\" notification method"
                   return 0
               }
               set syslog 1
            } elseif {($token2 == "snmp")} {
               if {$snmp == 1} {
                   puts "\n\nERROR: Duplicate \"snmp\" notification method"
                   return 0
               }
               set snmp 1
            } else {
               puts "$err $token2"
               return 0 
            }
        }
    } else {
        puts "$err $notif"
        return 0 
    }

    if {($email == 0) && ($snmp == 0) && ($syslog == 0)} {
        puts "$err $notif"
        return 0 
    }

    return 1 
}

#############################################################################
#    Description: check whether the policy name is already registered or not
#
#    IN:   name - policy name
#    OUT:  return 1 if policy is already registered.  Otherwise return 0
#
##############################################################################

proc is_policy_registered {name} {

    if {[catch {exec "show event manager policy register user | i $name"}\
        result ]} {
        puts "Error sending show policy registered command"
    } else {
        foreach line [split $result "\n"] {
            if {[string first $name $line] != -1} {
                puts "\n\nERROR: $name is already registered."
                return 1
            }
        }
    }
    return 0
}

#############################################################################
#    Description: validate the policy name passed to the registration script
#
#    IN:   policy name
#    OUT:  return 1 if validation is successful.  Otherwise return 0
#
##############################################################################

proc validate_policy_name {polname} {

    if {$polname == "cfm_ondemand.tcl"} {
        return 1
    } elseif {$polname == "cfm_cctimeout.tcl"} {
        return 1
    } elseif {$polname == "cfm_autotrace.tcl"} {
        return 1
    } elseif {$polname == "cfm_shut.tcl"} {
        return 1
    } elseif {$polname == "cfm_unshut.tcl"} {
        return 1
    } else {
        puts "\n\nERROR: Policy name is not valid"
        return 0
    }
}

#############################################################################
#    Description: validate the auto-trace timer value 
#
#    IN:   timer: autotrace timer 
#          hold_timer: Traceroute cache hold-timer
#    OUT:  return 1 if validation is successful.  Otherwise return 0
#
##############################################################################

proc validate_autotrace_timer {timer hold_timer} {

    set minimum_timer 60

    ### Validate timer 
    ### First, timer should be an integer
    if {[string is integer $timer ] != 1} {
        puts "\n\nERROR: Enter an integer number for auto-trace timer in minutes"
        puts "       You entered - $timer mins"
        return 0
    ### Make sure timer falls into the valid range
    } elseif {($timer < $minimum_timer) || ($timer > $hold_timer)} {
        puts "CFM auto-trace timer: $timer mins"
        puts "\nERROR: Timer value validation failure. Adjust timer values such that"
        puts "       1) auto-trace timer >= 60 mins"        
        puts "       2) hold-timer >= n * auto-trace timer where n >= 1,\
                        n = 1.5 recommended\n"
        return 0
    }  
    return 1
}

#############################################################################
#    Description: Retrieve CFM debug timer flag 
#
#    IN:   
#          
#    OUT: debug timer value - 0 or 1 
#
##############################################################################

proc get_debug_cfm_timer {} {

    set result [exec "show event manager environ _debug_cfm_timer"]

    if {[string first "No such environment" $result] >= 0} {
        return 0
    } else {
        return $result
    }
}


#    Execution of the script starts from here
#    Check if the number of arguments entered by user is correct otherwise send
#    error message. 
set cmd ""
global trace

### Set the debug for helper scripts ###
if {![info exists _debug_cfm_deploy]} {
    set _debug_cfm_deploy 0
}

set trace $_debug_cfm_deploy
set policy 0
set err "\n\nERROR: Incorrect number of arguments"

if {($argc >= 8)} {
    set script [lindex $argv 0]
    set notif2 [lindex $argv 1]
    set notif [string toupper $notif2]
    set config [lindex $argv 2]
    set config [string toupper $config]
    set event [lindex $argv 3]
    set event [string toupper $event]
    set command [lindex $argv 4]
    set command [string toupper $command]
    set prepend [lindex $argv 5]
    set prepend [string toupper $prepend]
    set user_pol_dir [lindex $argv 6]
    set user_lib_dir [lindex $argv 7]

    if {[validate_policy_name $script] == 0} {
        puts [get_cfm_usage all 0]
        return
    }

    if {[is_policy_registered $script] == 1} {
        return
    }

    if {[validate_notify $notif] == 0} {
        display_usage $script 
        return
    }

    if {[validate_common_flags $config $event $command $prepend] == 0} {
        display_usage $script 
        return
    }

    if {[sourceLibFile $user_lib_dir $user_pol_dir] == 0} {
        display_usage $script 
        return
    }

    unset_eem_dir "policy"
    unset_eem_dir "library"
    set_eem_dir policy $user_pol_dir
    set_eem_dir library $user_lib_dir

    if {[validateParameters $notif $config $event $command $prepend\
         $policy] == 1} {
        display_usage $script 
        return
    }
} elseif {$argc == 0} {
    if {$tcl_interactive != 1} {
        puts $err 
        puts [get_cfm_usage all 0]
        return
    }
} elseif {$argc == 1} {
    puts $err 
    display_usage [lindex $argv 0]
    return
} elseif {$argc == 2} {
    ### if policy and key are specified
    set policy [lindex $argv 0]
    set key [lindex $argv 1]
    if {[is_valid_policy $policy] == 1} {
        if {[is_valid_help_key $key] == 1} {
            puts [get_cfm_usage $policy 2]
        } elseif {[string match {-variables} $key] == 1} {
            display_cfm_variable $policy
            return
        } else {
            puts $err 
            puts [get_cfm_usage $policy 1]
        }
    } else {
        ### else display usage for all
        puts $err 
        puts [get_cfm_usage "all" 0]
    }
    return
} else {
    puts $err 
    display_usage [lindex $argv 0]
    return
}

switch  -- [lindex $argv 0] {

    cfm_ondemand.tcl {
        debug $trace "Validating CFM On-Demand policy parameters..."
        if {($argc ==  10) || ($argc == 11)} {

            ### validate domain ###
            set domain [lindex $argv 8]
            if {[validate_domain $domain $cmd $policy] == 0} {
                exit_script_with_errors "cfm_ondemand.tcl"
                return
            }

            ### validate ma name ###
            set ma_name [lindex $argv 9]
            if {[validate_ma_name $domain $ma_name $policy] == 0} {
                exit_script_with_errors "cfm_ondemand.tcl"
                return
            }

            ### validate MEP id ###
            if {$argc == 11} {
                set mpid [lindex $argv 10]
                if {[validate_mpid $mpid cmd $domain $policy $ma_name] == 0} {
                    exit_script_with_errors "cfm_ondemand.tcl"
                    return
                }
            } else {
                set mpid "NULL"
            }
            set code [ondemand_config $notif $config $event $command\
                                      $prepend $ma_name $domain $mpid]
        } else {
            incorrect_arguments_error $argc "cfm_ondemand.tcl"
        }
    }

    cfm_cctimeout.tcl {
        debug $trace "Validating CFM Cc-Timeout policy parameters..."
        if {$argc == 9} {
            set ma_file [lindex $argv 8]
            set ma_list {}
            if {[get_ma_list $ma_file ma_list] == 0} {
                exit_script_with_errors "cfm_cctimeout.tcl" 
                return
            }
            ### validate ma list from file ###
            if {[validate_ma_list $ma_list $policy cmd] == 0} {
                exit_script_with_errors "cfm_cctimeout.tcl"
                return
            }
            set code [cctimeout_config $notif $config $event $command \
                                       $prepend $ma_file] 
        } else {
            incorrect_arguments_error $argc "cfm_cctimeout.tcl"
        }
    }

    cfm_shut.tcl {
        debug $trace "Validating CFM Shut policy parameters..."
        if {$argc == 10} {
            set ma_file [lindex $argv 8]
            set ma_list {}
            if {[get_ma_list $ma_file ma_list] == 0} {
                exit_script_with_errors "cfm_shut.tcl" 
                return
            }
            if {[validate_ma_list $ma_list $policy cmd] == 0} {
                exit_script_with_errors "cfm_shut.tcl" 
                return
            }
            set p2p_service [string toupper [lindex $argv 9]]
            if {[validate_p2p $p2p_service] == 0} {
                exit_script_with_errors "cfm_shut.tcl" 
                return
            }
            set code [shut_config $notif $config $event \
                                   $command $prepend $ma_file $p2p_service] 
            if {$code == 1} {
                ### display MA warning message for the shut script
                display_shut_ma_warning_message
            }
        } else {
            incorrect_arguments_error $argc "cfm_shut.tcl"
        }
    }
    
    cfm_unshut.tcl {
        debug $trace "Validating CFM unshut policy parameters..."
        if {$argc == 8} {
            set code [unshut_config $notif $config $event $command $prepend] 
        } else {
            incorrect_arguments_error $argc "cfm_unshut.tcl"
        }
    }
    
    cfm_autotrace.tcl {
        debug $trace "Validating CFM auto-trace policy parameters..."
        if {($argc == 9) || ($argc == 10)} {
            set ma_file [lindex $argv 8]
            set ma_list {}
            if {[get_ma_list $ma_file ma_list] == 0} {
                exit_script_with_errors "cfm_autotrace.tcl" 
                return
            }
            if {[validate_ma_list $ma_list $policy cmd] == 0} {
                exit_script_with_errors "cfm_autotrace.tcl"
                return
            }
            ### default timer value is 6 hours.  The timer unit is in minutes
            ### 
            set timer 360 

            ### make sure the traceroute cache is enabled
            ### otherwise, return error and exit
            if {[is_traceroute_cache_enabled $policy cmd result] == 0} {
                set err "Traceroute cache is not enabled"
                exit_script_with_errors "cfm_autotrace.tcl" $err
                return
            }

            ### make sure the holder timer is 60 minutes over 
            ### auto-trace timer.  Otherwise, return errror and exit
            set hold_timer 100 
            set size 0
            if {[display_traceroute_configuration $policy cmd \
                                                   hold_timer size] ==0} {
                set err "Failed to parse traceroute configuration"
                exit_script_with_errors "cfm_autotrace.tcl" $err
                return
            }
            if {$argc == 10} {
		set timer [lindex $argv 9]
            } else {
                set msg1 "\nNo timer value is specified. " 
                set msg2 "Use default value: $timer mins\n"
                append msg1 $msg2
                puts $msg1  
            }
            if {[validate_autotrace_timer $timer $hold_timer] != 1 } {
                return
            }

            set _debug_cfm_timer [get_debug_cfm_timer]
            debug $trace "_debug_cfm_timer: $_debug_cfm_timer"

            ### if debug cfm timer value is zero, use unit in minutes
            ### if debug cfm timer is > 0, use unit in secs
            if {$_debug_cfm_timer == 0} {
                set timer2 [expr 60*$timer]
                debug $trace "timer value: $timer2 secs" 
            } else {
                debug $trace "timer value: $timer mins" 
                set timer2 $timer
            }
            if {$_debug_cfm_timer == 0} {
                puts "CFM autotrace timer: $timer mins"
            } else {
                puts "CFM autotrace timer: $timer2 secs"
            }
            set code [autotrace_config $notif $config $event \
                                       $command $prepend $ma_file $timer2] 
        } else {
            incorrect_arguments_error $argc "cfm_autotrace.tcl"
        }    
    }   

    default {
        puts "\n\nERROR: Invalid policy name [lindex $argv 0]"
        puts [get_cfm_usage "all" "FALSE" ] 
        return
    }     
}


#Cisco Tcl Signature V1.0
#3082077506092a864886f70d010702a082076630820762020101310b3009
#06052b0e03021a0500300b06092a864886f70d010701a08204bb308204b7
#3082039fa003020102020900d48b1bbfb069e9b9300d06092a864886f70d
#0101050500308198310b3009060355040613024341311230100603550408
#130963616c69666f6e69613111300f060355040713086d69706c69746173
#311b3019060355040a1312636973636f2073797374656d7320696e632e31
#0e300c060355040b13054e53535447311630140603550403130d43687269
#7374696e652059616e311d301b06092a864886f70d010901160e6379616e
#40636973636f2e636f6d301e170d3037313231383138333232375a170d31
#30313231373138333232375a308198310b30090603550406130243413112
#30100603550408130963616c69666f6e69613111300f060355040713086d
#69706c69746173311b3019060355040a1312636973636f2073797374656d
#7320696e632e310e300c060355040b13054e535354473116301406035504
#03130d436872697374696e652059616e311d301b06092a864886f70d0109
#01160e6379616e40636973636f2e636f6d30820122300d06092a864886f7
#0d01010105000382010f003082010a02820101009c22f1d01af397756ab6
#cfce92a9e761b73fbf1efd070aa4e90448e6077e2c127ff47a5951de7790
#55da014ff152576a08d7199b8ad5757eb8c9a7c1a493715df2176e9f56f0
#74735d43420cf1ed17fe0aa3494724808aefcf274572c7750be071a94560
#9c045c48a5454f9e7933bf93d967bcf582918b1bbd48f47dd978c7ea9c88
#23eb01482a40ef770f823ea2e63eaf7cab5367bd51b54979925464e27b22
#82fc5bfe94f6f41f2f91e56444ae371a050107fe79fd5db3f228bbbd5fcb
#653a87a3c1268d74500ba1489382290804d9d7937d3269b1c55eae51f9f7
#62eadbd4226ba6d4008414b353181ee6cfbe24204e4a82436b117c255014
#77ec74c3ac4d0203010001a38201003081fd301d0603551d0e041604149c
#0e12d3a222891fc1300b780f38b54e07e228073081cd0603551d230481c5
#3081c280149c0e12d3a222891fc1300b780f38b54e07e22807a1819ea481
#9b308198310b300906035504061302434131123010060355040813096361
#6c69666f6e69613111300f060355040713086d69706c69746173311b3019
#060355040a1312636973636f2073797374656d7320696e632e310e300c06
#0355040b13054e53535447311630140603550403130d436872697374696e
#652059616e311d301b06092a864886f70d010901160e6379616e40636973
#636f2e636f6d820900d48b1bbfb069e9b9300c0603551d13040530030101
#ff300d06092a864886f70d010105050003820101003e6d657e30c21acb30
#d7e91aef99cd6f83a8644d7ae437fb4a0cd1a8c6a0f1cb2194be80868b75
#1d20ee8dc72a1f80de697928820ec3f806cec67df136d6f8b6ff0ba7ef6f
#e4c228b37e7407becff9903198ebe2258d3774f198de8d0366cb10496408
#57dff62f247dfcae9813fde4167c1f959b291490cfb8e0b8db3395ce019f
#07c94aa4a86d1a0dae2b6cc6d139db740841dc65add58512ea59336dbc8c
#9abc14d500e42c8ea5031c201eec027d14e9878974629d2fdbe0d0cfffab
#8ab73f148bdec788de59550443ccf12cb3e74518274c3d7d2892eeb90f23
#0d5e853b85542c05b12198c097c2fb4023b370f0629645e11552f5429fb8
#aacef60a9344b5318202823082027e0201013081a6308198310b30090603
#55040613024341311230100603550408130963616c69666f6e6961311130
#0f060355040713086d69706c69746173311b3019060355040a1312636973
#636f2073797374656d7320696e632e310e300c060355040b13054e535354
#47311630140603550403130d436872697374696e652059616e311d301b06
#092a864886f70d010901160e6379616e40636973636f2e636f6d020900d4
#8b1bbfb069e9b9300906052b0e03021a0500a081b1301806092a864886f7
#0d010903310b06092a864886f70d010701301c06092a864886f70d010905
#310f170d3039303631373232303930365a302306092a864886f70d010904
#31160414d3830e756a2c8272712e5a42a5a1c709cacf4b38305206092a86
#4886f70d01090f31453043300a06082a864886f70d0307300e06082a8648
#86f70d030202020080300d06082a864886f70d0302020140300706052b0e
#030207300d06082a864886f70d0302020128300d06092a864886f70d0101
#010500048201007ca9137f65f1a328863518def65c66b55b5610bb88ebd8
#94dd2b3e1d4c51d9230ca9adf4c71b845bb8a675fb3b13e7f499f94cafea
#8048262ed50f66de822826f0fd3229fe7d49eaa364d20510fe6918bd066a
#4f1ce284e2d07b1737fda7871d66cb107454c7d37357781d483579e4d411
#019c2cee1ef879eb9cd44edf236bab038d175d237185519c6bc2f7bf8bd6
#0cbf3e1aff93f652c81ddc449592603d6ff924d00bc1026baf068f5060aa
#39397d5d3d8acc10a809f8d40a47b8f5b62ae425566050cf587c6fd9fa6d
#3864e1ed986da544c7ad252fe83e2913a1a7a201d5a773221a17ce44ff37
#9a422caaca822785a7d24f1946d7cffbc45d67341605e3
